# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'tiff_io_pytemplate.ui'
#
# Created by: PyQt5 UI code generator 5.9.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_DockWidget(object):
    def setupUi(self, DockWidget):
        DockWidget.setObjectName("DockWidget")
        DockWidget.resize(600, 411)
        DockWidget.setFloating(True)
        self.dockWidgetContents = QtWidgets.QWidget()
        self.dockWidgetContents.setObjectName("dockWidgetContents")
        self.verticalLayout_6 = QtWidgets.QVBoxLayout(self.dockWidgetContents)
        self.verticalLayout_6.setObjectName("verticalLayout_6")
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout()
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.label = QtWidgets.QLabel(self.dockWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(14)
        font.setBold(True)
        font.setWeight(75)
        self.label.setFont(font)
        self.label.setObjectName("label")
        self.verticalLayout_4.addWidget(self.label)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.btnSelectTiff = QtWidgets.QPushButton(self.dockWidgetContents)
        self.btnSelectTiff.setMaximumSize(QtCore.QSize(90, 32))
        self.btnSelectTiff.setObjectName("btnSelectTiff")
        self.horizontalLayout.addWidget(self.btnSelectTiff)
        self.labelFileTiff = QtWidgets.QLabel(self.dockWidgetContents)
        self.labelFileTiff.setText("")
        self.labelFileTiff.setTextInteractionFlags(QtCore.Qt.LinksAccessibleByMouse|QtCore.Qt.TextSelectableByKeyboard|QtCore.Qt.TextSelectableByMouse)
        self.labelFileTiff.setObjectName("labelFileTiff")
        self.horizontalLayout.addWidget(self.labelFileTiff)
        self.verticalLayout_4.addLayout(self.horizontalLayout)
        self.groupBox = QtWidgets.QGroupBox(self.dockWidgetContents)
        self.groupBox.setObjectName("groupBox")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.groupBox)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.radioButtonAsArray = QtWidgets.QRadioButton(self.groupBox)
        self.radioButtonAsArray.setObjectName("radioButtonAsArray")
        self.verticalLayout_2.addWidget(self.radioButtonAsArray)
        self.radioButtonAsArrayMulti = QtWidgets.QRadioButton(self.groupBox)
        self.radioButtonAsArrayMulti.setObjectName("radioButtonAsArrayMulti")
        self.verticalLayout_2.addWidget(self.radioButtonAsArrayMulti)
        self.radioButtonImread = QtWidgets.QRadioButton(self.groupBox)
        self.radioButtonImread.setObjectName("radioButtonImread")
        self.verticalLayout_2.addWidget(self.radioButtonImread)
        self.verticalLayout_4.addWidget(self.groupBox)
        self.groupBox_2 = QtWidgets.QGroupBox(self.dockWidgetContents)
        self.groupBox_2.setObjectName("groupBox_2")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.groupBox_2)
        self.verticalLayout.setObjectName("verticalLayout")
        self.radioButton_axes_default = QtWidgets.QRadioButton(self.groupBox_2)
        self.radioButton_axes_default.setChecked(True)
        self.radioButton_axes_default.setObjectName("radioButton_axes_default")
        self.verticalLayout.addWidget(self.radioButton_axes_default)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.radioButton_axes_custom = QtWidgets.QRadioButton(self.groupBox_2)
        self.radioButton_axes_custom.setText("")
        self.radioButton_axes_custom.setObjectName("radioButton_axes_custom")
        self.horizontalLayout_2.addWidget(self.radioButton_axes_custom)
        self.lineEdit_axes_custom = QtWidgets.QLineEdit(self.groupBox_2)
        self.lineEdit_axes_custom.setText("")
        self.lineEdit_axes_custom.setMaxLength(4)
        self.lineEdit_axes_custom.setObjectName("lineEdit_axes_custom")
        self.horizontalLayout_2.addWidget(self.lineEdit_axes_custom)
        self.verticalLayout.addLayout(self.horizontalLayout_2)
        self.verticalLayout_4.addWidget(self.groupBox_2)
        self.horizontalLayout_4.addLayout(self.verticalLayout_4)
        self.line = QtWidgets.QFrame(self.dockWidgetContents)
        self.line.setFrameShape(QtWidgets.QFrame.VLine)
        self.line.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line.setObjectName("line")
        self.horizontalLayout_4.addWidget(self.line)
        self.verticalLayout_5 = QtWidgets.QVBoxLayout()
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.label_2 = QtWidgets.QLabel(self.dockWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(14)
        font.setBold(True)
        font.setWeight(75)
        self.label_2.setFont(font)
        self.label_2.setObjectName("label_2")
        self.verticalLayout_5.addWidget(self.label_2)
        self.groupBox_meta_loader = QtWidgets.QGroupBox(self.dockWidgetContents)
        self.groupBox_meta_loader.setCheckable(True)
        self.groupBox_meta_loader.setObjectName("groupBox_meta_loader")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.groupBox_meta_loader)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.btnSelectMetaFile = QtWidgets.QPushButton(self.groupBox_meta_loader)
        self.btnSelectMetaFile.setMaximumSize(QtCore.QSize(130, 32))
        self.btnSelectMetaFile.setObjectName("btnSelectMetaFile")
        self.horizontalLayout_3.addWidget(self.btnSelectMetaFile)
        self.labelFileMeta = QtWidgets.QLabel(self.groupBox_meta_loader)
        self.labelFileMeta.setText("")
        self.labelFileMeta.setTextInteractionFlags(QtCore.Qt.LinksAccessibleByMouse|QtCore.Qt.TextSelectableByKeyboard|QtCore.Qt.TextSelectableByMouse)
        self.labelFileMeta.setObjectName("labelFileMeta")
        self.horizontalLayout_3.addWidget(self.labelFileMeta)
        self.verticalLayout_3.addLayout(self.horizontalLayout_3)
        self.label_3 = QtWidgets.QLabel(self.groupBox_meta_loader)
        self.label_3.setObjectName("label_3")
        self.verticalLayout_3.addWidget(self.label_3)
        self.listWidget_meta_data_loader = QtWidgets.QListWidget(self.groupBox_meta_loader)
        self.listWidget_meta_data_loader.setObjectName("listWidget_meta_data_loader")
        self.verticalLayout_3.addWidget(self.listWidget_meta_data_loader)
        self.verticalLayout_5.addWidget(self.groupBox_meta_loader)
        self.horizontalLayout_4.addLayout(self.verticalLayout_5)
        self.verticalLayout_6.addLayout(self.horizontalLayout_4)
        self.btnLoadIntoWorkEnv = QtWidgets.QPushButton(self.dockWidgetContents)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.MinimumExpanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.btnLoadIntoWorkEnv.sizePolicy().hasHeightForWidth())
        self.btnLoadIntoWorkEnv.setSizePolicy(sizePolicy)
        self.btnLoadIntoWorkEnv.setObjectName("btnLoadIntoWorkEnv")
        self.verticalLayout_6.addWidget(self.btnLoadIntoWorkEnv)
        DockWidget.setWidget(self.dockWidgetContents)

        self.retranslateUi(DockWidget)
        QtCore.QMetaObject.connectSlotsByName(DockWidget)
        DockWidget.setTabOrder(self.btnSelectTiff, self.radioButtonAsArray)
        DockWidget.setTabOrder(self.radioButtonAsArray, self.radioButtonAsArrayMulti)
        DockWidget.setTabOrder(self.radioButtonAsArrayMulti, self.radioButtonImread)
        DockWidget.setTabOrder(self.radioButtonImread, self.radioButton_axes_default)
        DockWidget.setTabOrder(self.radioButton_axes_default, self.radioButton_axes_custom)
        DockWidget.setTabOrder(self.radioButton_axes_custom, self.lineEdit_axes_custom)
        DockWidget.setTabOrder(self.lineEdit_axes_custom, self.groupBox_meta_loader)
        DockWidget.setTabOrder(self.groupBox_meta_loader, self.btnSelectMetaFile)
        DockWidget.setTabOrder(self.btnSelectMetaFile, self.listWidget_meta_data_loader)
        DockWidget.setTabOrder(self.listWidget_meta_data_loader, self.btnLoadIntoWorkEnv)

    def retranslateUi(self, DockWidget):
        _translate = QtCore.QCoreApplication.translate
        DockWidget.setWindowTitle(_translate("DockWidget", "Tiff file &I/O"))
        self.label.setText(_translate("DockWidget", "Tiff File"))
        self.btnSelectTiff.setText(_translate("DockWidget", "Select tiff file"))
        self.groupBox.setToolTip(_translate("DockWidget", "The options for “Load Method” correspond to the tifffile library method that is used for loading the images"))
        self.groupBox.setTitle(_translate("DockWidget", "Image Load Method:"))
        self.radioButtonAsArray.setToolTip(_translate("DockWidget", "Usually faster alternative to imread"))
        self.radioButtonAsArray.setText(_translate("DockWidget", "asarray"))
        self.radioButtonAsArrayMulti.setToolTip(_translate("DockWidget", "Use if tiff was created in append mode"))
        self.radioButtonAsArrayMulti.setText(_translate("DockWidget", "asarray - multi series"))
        self.radioButtonImread.setToolTip(_translate("DockWidget", "Should work for most tiffs if they were not created in append mode."))
        self.radioButtonImread.setText(_translate("DockWidget", "i&mread"))
        self.groupBox_2.setTitle(_translate("DockWidget", "Image Axes order"))
        self.radioButton_axes_default.setToolTip(_translate("DockWidget", "default option, usual dimensions"))
        self.radioButton_axes_default.setText(_translate("DockWidget", "&2D: [t, x, y]\n"
"3D: [t, z, x, y]"))
        self.lineEdit_axes_custom.setPlaceholderText(_translate("DockWidget", "Custom, ex: xyzt"))
        self.label_2.setText(_translate("DockWidget", "Meta data file"))
        self.groupBox_meta_loader.setToolTip(_translate("DockWidget", "Select a meta data loader that corresonds to your meta data file.\n"
"See the docs for instructions on how to create your own loader."))
        self.groupBox_meta_loader.setTitle(_translate("DockWidget", "&Load meta data"))
        self.btnSelectMetaFile.setText(_translate("DockWidget", "Select file"))
        self.label_3.setText(_translate("DockWidget", "Meta data format:"))
        self.btnLoadIntoWorkEnv.setText(_translate("DockWidget", "Load into Work Environment"))

