# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file './project_config_pytemplate.ui'
#
# Created by: PyQt5 UI code generator 5.9.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_Form(object):
    def setupUi(self, Form):
        Form.setObjectName("Form")
        Form.resize(795, 610)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(Form.sizePolicy().hasHeightForWidth())
        Form.setSizePolicy(sizePolicy)
        Form.setMinimumSize(QtCore.QSize(593, 610))
        Form.setMaximumSize(QtCore.QSize(2000, 2000))
        self.gridLayout = QtWidgets.QGridLayout(Form)
        self.gridLayout.setObjectName("gridLayout")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout()
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        spacerItem = QtWidgets.QSpacerItem(1, 10, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem)
        self.label_4 = QtWidgets.QLabel(Form)
        self.label_4.setObjectName("label_4")
        self.horizontalLayout_3.addWidget(self.label_4)
        spacerItem1 = QtWidgets.QSpacerItem(1, 10, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem1)
        self.verticalLayout_2.addLayout(self.horizontalLayout_3)
        self.listwROIDefs = QtWidgets.QListWidget(Form)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.listwROIDefs.sizePolicy().hasHeightForWidth())
        self.listwROIDefs.setSizePolicy(sizePolicy)
        self.listwROIDefs.setAlternatingRowColors(True)
        self.listwROIDefs.setObjectName("listwROIDefs")
        self.verticalLayout_2.addWidget(self.listwROIDefs)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.lineEdNewROIDef = QtWidgets.QLineEdit(Form)
        self.lineEdNewROIDef.setObjectName("lineEdNewROIDef")
        self.horizontalLayout_2.addWidget(self.lineEdNewROIDef)
        self.btnAddNewROICol = QtWidgets.QPushButton(Form)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.btnAddNewROICol.sizePolicy().hasHeightForWidth())
        self.btnAddNewROICol.setSizePolicy(sizePolicy)
        self.btnAddNewROICol.setMaximumSize(QtCore.QSize(51, 16777215))
        self.btnAddNewROICol.setObjectName("btnAddNewROICol")
        self.horizontalLayout_2.addWidget(self.btnAddNewROICol)
        self.verticalLayout_2.addLayout(self.horizontalLayout_2)
        self.gridLayout.addLayout(self.verticalLayout_2, 0, 2, 1, 1)
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.gridLayout_3 = QtWidgets.QGridLayout()
        self.gridLayout_3.setObjectName("gridLayout_3")
        spacerItem2 = QtWidgets.QSpacerItem(1, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_3.addItem(spacerItem2, 0, 0, 1, 1)
        self.label_2 = QtWidgets.QLabel(Form)
        self.label_2.setObjectName("label_2")
        self.gridLayout_3.addWidget(self.label_2, 0, 1, 1, 1)
        spacerItem3 = QtWidgets.QSpacerItem(1, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_3.addItem(spacerItem3, 0, 2, 1, 1)
        self.listwInclude = QtWidgets.QListWidget(Form)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.listwInclude.sizePolicy().hasHeightForWidth())
        self.listwInclude.setSizePolicy(sizePolicy)
        self.listwInclude.setDragEnabled(True)
        self.listwInclude.setDragDropOverwriteMode(False)
        self.listwInclude.setDragDropMode(QtWidgets.QAbstractItemView.DragDrop)
        self.listwInclude.setDefaultDropAction(QtCore.Qt.MoveAction)
        self.listwInclude.setAlternatingRowColors(True)
        self.listwInclude.setSelectionMode(QtWidgets.QAbstractItemView.ExtendedSelection)
        self.listwInclude.setObjectName("listwInclude")
        self.gridLayout_3.addWidget(self.listwInclude, 1, 0, 1, 3)
        self.horizontalLayout_5.addLayout(self.gridLayout_3)
        self.gridLayout_4 = QtWidgets.QGridLayout()
        self.gridLayout_4.setObjectName("gridLayout_4")
        spacerItem4 = QtWidgets.QSpacerItem(1, 10, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_4.addItem(spacerItem4, 0, 2, 1, 1)
        self.label_3 = QtWidgets.QLabel(Form)
        self.label_3.setObjectName("label_3")
        self.gridLayout_4.addWidget(self.label_3, 0, 1, 1, 1)
        spacerItem5 = QtWidgets.QSpacerItem(1, 10, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_4.addItem(spacerItem5, 0, 0, 1, 1)
        self.listwExclude = QtWidgets.QListWidget(Form)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.listwExclude.sizePolicy().hasHeightForWidth())
        self.listwExclude.setSizePolicy(sizePolicy)
        self.listwExclude.setDragEnabled(True)
        self.listwExclude.setDragDropOverwriteMode(False)
        self.listwExclude.setDragDropMode(QtWidgets.QAbstractItemView.DragDrop)
        self.listwExclude.setDefaultDropAction(QtCore.Qt.MoveAction)
        self.listwExclude.setAlternatingRowColors(True)
        self.listwExclude.setSelectionMode(QtWidgets.QAbstractItemView.ExtendedSelection)
        self.listwExclude.setObjectName("listwExclude")
        self.gridLayout_4.addWidget(self.listwExclude, 1, 0, 1, 3)
        self.horizontalLayout_5.addLayout(self.gridLayout_4)
        self.gridLayout.addLayout(self.horizontalLayout_5, 0, 0, 1, 2)
        self.groupBox = QtWidgets.QGroupBox(Form)
        self.groupBox.setAlignment(QtCore.Qt.AlignCenter)
        self.groupBox.setObjectName("groupBox")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.groupBox)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.btnAddCustomColumn = QtWidgets.QPushButton(self.groupBox)
        self.btnAddCustomColumn.setObjectName("btnAddCustomColumn")
        self.gridLayout_2.addWidget(self.btnAddCustomColumn, 7, 0, 1, 1)
        self.radioButtonStr = QtWidgets.QRadioButton(self.groupBox)
        self.radioButtonStr.setObjectName("radioButtonStr")
        self.gridLayout_2.addWidget(self.radioButtonStr, 2, 0, 1, 1)
        self.label_7 = QtWidgets.QLabel(self.groupBox)
        self.label_7.setObjectName("label_7")
        self.gridLayout_2.addWidget(self.label_7, 1, 0, 1, 1)
        self.radioButtonInt64 = QtWidgets.QRadioButton(self.groupBox)
        self.radioButtonInt64.setObjectName("radioButtonInt64")
        self.gridLayout_2.addWidget(self.radioButtonInt64, 3, 0, 1, 1)
        self.radioButtonFloat64 = QtWidgets.QRadioButton(self.groupBox)
        self.radioButtonFloat64.setObjectName("radioButtonFloat64")
        self.gridLayout_2.addWidget(self.radioButtonFloat64, 4, 0, 1, 1)
        self.lineEditCustomColumnReplacementValue = QtWidgets.QLineEdit(self.groupBox)
        self.lineEditCustomColumnReplacementValue.setText("")
        self.lineEditCustomColumnReplacementValue.setObjectName("lineEditCustomColumnReplacementValue")
        self.gridLayout_2.addWidget(self.lineEditCustomColumnReplacementValue, 6, 0, 1, 1)
        self.lineEditCustomColumnName = QtWidgets.QLineEdit(self.groupBox)
        self.lineEditCustomColumnName.setObjectName("lineEditCustomColumnName")
        self.gridLayout_2.addWidget(self.lineEditCustomColumnName, 0, 0, 1, 1)
        self.radioButtonBoolean = QtWidgets.QRadioButton(self.groupBox)
        self.radioButtonBoolean.setEnabled(False)
        self.radioButtonBoolean.setObjectName("radioButtonBoolean")
        self.gridLayout_2.addWidget(self.radioButtonBoolean, 5, 0, 1, 1)
        self.gridLayout.addWidget(self.groupBox, 2, 2, 1, 1)
        self.horizontalLayout_8 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_8.setObjectName("horizontalLayout_8")
        spacerItem6 = QtWidgets.QSpacerItem(338, 13, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_8.addItem(spacerItem6)
        self.horizontalLayout_6 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_6.setObjectName("horizontalLayout_6")
        self.btnClose = QtWidgets.QPushButton(Form)
        self.btnClose.setObjectName("btnClose")
        self.horizontalLayout_6.addWidget(self.btnClose)
        self.btnSave = QtWidgets.QPushButton(Form)
        self.btnSave.setMinimumSize(QtCore.QSize(92, 0))
        self.btnSave.setObjectName("btnSave")
        self.horizontalLayout_6.addWidget(self.btnSave)
        self.horizontalLayout_8.addLayout(self.horizontalLayout_6)
        self.gridLayout.addLayout(self.horizontalLayout_8, 3, 0, 1, 3)
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        spacerItem7 = QtWidgets.QSpacerItem(1, 10, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_4.addItem(spacerItem7)
        self.label_5 = QtWidgets.QLabel(Form)
        self.label_5.setObjectName("label_5")
        self.horizontalLayout_4.addWidget(self.label_5)
        spacerItem8 = QtWidgets.QSpacerItem(1, 10, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_4.addItem(spacerItem8)
        self.verticalLayout.addLayout(self.horizontalLayout_4)
        self.listwStimDefs = QtWidgets.QListWidget(Form)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.listwStimDefs.sizePolicy().hasHeightForWidth())
        self.listwStimDefs.setSizePolicy(sizePolicy)
        self.listwStimDefs.setAlternatingRowColors(True)
        self.listwStimDefs.setObjectName("listwStimDefs")
        self.verticalLayout.addWidget(self.listwStimDefs)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.lineEdNewStimCol = QtWidgets.QLineEdit(Form)
        self.lineEdNewStimCol.setObjectName("lineEdNewStimCol")
        self.horizontalLayout.addWidget(self.lineEdNewStimCol)
        self.btnAddNewStimCol = QtWidgets.QPushButton(Form)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.btnAddNewStimCol.sizePolicy().hasHeightForWidth())
        self.btnAddNewStimCol.setSizePolicy(sizePolicy)
        self.btnAddNewStimCol.setMaximumSize(QtCore.QSize(51, 16777215))
        self.btnAddNewStimCol.setObjectName("btnAddNewStimCol")
        self.horizontalLayout.addWidget(self.btnAddNewStimCol)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.gridLayout.addLayout(self.verticalLayout, 2, 0, 1, 1)
        self.verticalLayout_3 = QtWidgets.QVBoxLayout()
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.horizontalLayout_9 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_9.setObjectName("horizontalLayout_9")
        spacerItem9 = QtWidgets.QSpacerItem(1, 10, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_9.addItem(spacerItem9)
        self.label_6 = QtWidgets.QLabel(Form)
        self.label_6.setObjectName("label_6")
        self.horizontalLayout_9.addWidget(self.label_6)
        spacerItem10 = QtWidgets.QSpacerItem(1, 10, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_9.addItem(spacerItem10)
        self.verticalLayout_3.addLayout(self.horizontalLayout_9)
        self.listwCustomColumns = QtWidgets.QListWidget(Form)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.listwCustomColumns.sizePolicy().hasHeightForWidth())
        self.listwCustomColumns.setSizePolicy(sizePolicy)
        self.listwCustomColumns.setAlternatingRowColors(True)
        self.listwCustomColumns.setObjectName("listwCustomColumns")
        self.verticalLayout_3.addWidget(self.listwCustomColumns)
        self.gridLayout.addLayout(self.verticalLayout_3, 2, 1, 1, 1)
        self.line = QtWidgets.QFrame(Form)
        self.line.setFrameShape(QtWidgets.QFrame.HLine)
        self.line.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line.setObjectName("line")
        self.gridLayout.addWidget(self.line, 1, 0, 1, 3)

        self.retranslateUi(Form)
        self.lineEdNewROIDef.returnPressed.connect(self.btnAddNewROICol.click)
        self.lineEdNewStimCol.returnPressed.connect(self.btnAddNewStimCol.click)
        QtCore.QMetaObject.connectSlotsByName(Form)
        Form.setTabOrder(self.listwInclude, self.listwExclude)
        Form.setTabOrder(self.listwExclude, self.listwROIDefs)
        Form.setTabOrder(self.listwROIDefs, self.listwStimDefs)
        Form.setTabOrder(self.listwStimDefs, self.lineEdNewROIDef)
        Form.setTabOrder(self.lineEdNewROIDef, self.btnAddNewROICol)
        Form.setTabOrder(self.btnAddNewROICol, self.lineEdNewStimCol)
        Form.setTabOrder(self.lineEdNewStimCol, self.btnAddNewStimCol)
        Form.setTabOrder(self.btnAddNewStimCol, self.btnSave)
        Form.setTabOrder(self.btnSave, self.btnClose)

    def retranslateUi(self, Form):
        _translate = QtCore.QCoreApplication.translate
        Form.setWindowTitle(_translate("Form", "Dataframe Config"))
        self.label_4.setText(_translate("Form", "ROI Type Columns"))
        self.lineEdNewROIDef.setPlaceholderText(_translate("Form", "Enter New ROI Column Name"))
        self.btnAddNewROICol.setText(_translate("Form", "Add"))
        self.label_2.setText(_translate("Form", "Visible in Project Browser"))
        self.label_3.setText(_translate("Form", "Hidden in Project Browser"))
        self.groupBox.setTitle(_translate("Form", "Add new custom column"))
        self.btnAddCustomColumn.setText(_translate("Form", "Add"))
        self.radioButtonStr.setText(_translate("Form", "standard te&xt (str)"))
        self.label_7.setText(_translate("Form", "Choose data type:"))
        self.radioButtonInt64.setText(_translate("Form", "whole &numbers (np.int64)"))
        self.radioButtonFloat64.setText(_translate("Form", "deci&mal numbers (np.float64)"))
        self.lineEditCustomColumnReplacementValue.setToolTip(_translate("Form", "If the project dataframe is not empty this value will be set in all rows of the new column"))
        self.lineEditCustomColumnReplacementValue.setPlaceholderText(_translate("Form", "Dataframe replacement value"))
        self.lineEditCustomColumnName.setToolTip(_translate("Form", "Name for the new column"))
        self.lineEditCustomColumnName.setPlaceholderText(_translate("Form", "Column name"))
        self.radioButtonBoolean.setText(_translate("Form", "boolean (True/False &values)"))
        self.btnClose.setText(_translate("Form", "Close"))
        self.btnSave.setText(_translate("Form", "Save and apply"))
        self.label_5.setText(_translate("Form", "Stimulus/Behavior Type Columns"))
        self.lineEdNewStimCol.setPlaceholderText(_translate("Form", "Enter New Stim Column Name"))
        self.btnAddNewStimCol.setText(_translate("Form", "Add"))
        self.label_6.setText(_translate("Form", "Custom Columns"))

