# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file './control_widget.ui'
#
# Created by: PyQt5 UI code generator 5.9.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_ControlWidget(object):
    def setupUi(self, ControlWidget):
        ControlWidget.setObjectName("ControlWidget")
        ControlWidget.resize(263, 680)
        self.dockWidgetContents = QtWidgets.QWidget()
        self.dockWidgetContents.setObjectName("dockWidgetContents")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.dockWidgetContents)
        self.verticalLayout.setObjectName("verticalLayout")
        self.label = QtWidgets.QLabel(self.dockWidgetContents)
        self.label.setObjectName("label")
        self.verticalLayout.addWidget(self.label)
        self.listWidgetDataColumns = ListWidget(self.dockWidgetContents)
        self.listWidgetDataColumns.setObjectName("listWidgetDataColumns")
        self.verticalLayout.addWidget(self.listWidgetDataColumns)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.label_2 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_2.setObjectName("label_2")
        self.horizontalLayout.addWidget(self.label_2)
        self.comboBoxGroupBy = ComboBox(self.dockWidgetContents)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.comboBoxGroupBy.sizePolicy().hasHeightForWidth())
        self.comboBoxGroupBy.setSizePolicy(sizePolicy)
        self.comboBoxGroupBy.setObjectName("comboBoxGroupBy")
        self.horizontalLayout.addWidget(self.comboBoxGroupBy)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.label_7 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_7.setObjectName("label_7")
        self.verticalLayout.addWidget(self.label_7)
        self.listWidgetColormap = ColormapListWidget(self.dockWidgetContents)
        self.listWidgetColormap.setObjectName("listWidgetColormap")
        self.verticalLayout.addWidget(self.listWidgetColormap)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.label_4 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_4.setObjectName("label_4")
        self.horizontalLayout_2.addWidget(self.label_4)
        self.comboBoxUUIDColumn = ComboBox(self.dockWidgetContents)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.comboBoxUUIDColumn.sizePolicy().hasHeightForWidth())
        self.comboBoxUUIDColumn.setSizePolicy(sizePolicy)
        self.comboBoxUUIDColumn.setObjectName("comboBoxUUIDColumn")
        self.horizontalLayout_2.addWidget(self.comboBoxUUIDColumn)
        self.verticalLayout.addLayout(self.horizontalLayout_2)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.label_9 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_9.setObjectName("label_9")
        self.horizontalLayout_3.addWidget(self.label_9)
        self.comboBoxDPTCurve = ComboBox(self.dockWidgetContents)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.comboBoxDPTCurve.sizePolicy().hasHeightForWidth())
        self.comboBoxDPTCurve.setSizePolicy(sizePolicy)
        self.comboBoxDPTCurve.setObjectName("comboBoxDPTCurve")
        self.horizontalLayout_3.addWidget(self.comboBoxDPTCurve)
        self.verticalLayout.addLayout(self.horizontalLayout_3)
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.label_5 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_5.setObjectName("label_5")
        self.horizontalLayout_5.addWidget(self.label_5)
        self.spinBoxSpotSize = QtWidgets.QSpinBox(self.dockWidgetContents)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.spinBoxSpotSize.sizePolicy().hasHeightForWidth())
        self.spinBoxSpotSize.setSizePolicy(sizePolicy)
        self.spinBoxSpotSize.setMinimum(1)
        self.spinBoxSpotSize.setMaximum(999)
        self.spinBoxSpotSize.setSingleStep(1)
        self.spinBoxSpotSize.setProperty("value", 10)
        self.spinBoxSpotSize.setObjectName("spinBoxSpotSize")
        self.horizontalLayout_5.addWidget(self.spinBoxSpotSize)
        self.verticalLayout.addLayout(self.horizontalLayout_5)
        self.checkBoxLiveUpdateFromFlowchart = QtWidgets.QCheckBox(self.dockWidgetContents)
        self.checkBoxLiveUpdateFromFlowchart.setChecked(False)
        self.checkBoxLiveUpdateFromFlowchart.setObjectName("checkBoxLiveUpdateFromFlowchart")
        self.verticalLayout.addWidget(self.checkBoxLiveUpdateFromFlowchart)
        self.pushButtonUpdatePlot = QtWidgets.QPushButton(self.dockWidgetContents)
        self.pushButtonUpdatePlot.setObjectName("pushButtonUpdatePlot")
        self.verticalLayout.addWidget(self.pushButtonUpdatePlot)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.pushButtonSave = QtWidgets.QPushButton(self.dockWidgetContents)
        self.pushButtonSave.setObjectName("pushButtonSave")
        self.horizontalLayout_4.addWidget(self.pushButtonSave)
        self.pushButtonLoad = QtWidgets.QPushButton(self.dockWidgetContents)
        self.pushButtonLoad.setObjectName("pushButtonLoad")
        self.horizontalLayout_4.addWidget(self.pushButtonLoad)
        self.verticalLayout.addLayout(self.horizontalLayout_4)
        ControlWidget.setWidget(self.dockWidgetContents)

        self.retranslateUi(ControlWidget)
        QtCore.QMetaObject.connectSlotsByName(ControlWidget)

    def retranslateUi(self, ControlWidget):
        _translate = QtCore.QCoreApplication.translate
        ControlWidget.setWindowTitle(_translate("ControlWidget", "Co&ntrols"))
        self.label.setText(_translate("ControlWidget", "Data Column(s)"))
        self.label_2.setText(_translate("ControlWidget", "Group by"))
        self.label_7.setText(_translate("ControlWidget", "Choose colormap:"))
        self.label_4.setText(_translate("ControlWidget", "UUID column"))
        self.label_9.setText(_translate("ControlWidget", "DPT Curve column"))
        self.label_5.setText(_translate("ControlWidget", "Spot size"))
        self.checkBoxLiveUpdateFromFlowchart.setText(_translate("ControlWidget", "Live update data from flowchart"))
        self.pushButtonUpdatePlot.setText(_translate("ControlWidget", "Update Plot"))
        self.pushButtonSave.setText(_translate("ControlWidget", "Save"))
        self.pushButtonLoad.setText(_translate("ControlWidget", "Load"))

from ....pyqtgraphCore.widgets.ComboBox import ComboBox
from ....pyqtgraphCore.widgets.ListWidget import ListWidget
from ...utils import ColormapListWidget
