# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file './controls.ui'
#
# Created by: PyQt5 UI code generator 5.9.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_DockWidget(object):
    def setupUi(self, DockWidget):
        DockWidget.setObjectName("DockWidget")
        DockWidget.resize(267, 690)
        DockWidget.setMinimumSize(QtCore.QSize(96, 690))
        font = QtGui.QFont()
        font.setBold(False)
        font.setWeight(50)
        DockWidget.setFont(font)
        self.dockWidgetContents = QtWidgets.QWidget()
        self.dockWidgetContents.setObjectName("dockWidgetContents")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.dockWidgetContents)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.toolBox = QtWidgets.QToolBox(self.dockWidgetContents)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.toolBox.setFont(font)
        self.toolBox.setObjectName("toolBox")
        self.page = QtWidgets.QWidget()
        self.page.setGeometry(QtCore.QRect(0, 0, 251, 578))
        font = QtGui.QFont()
        font.setBold(False)
        font.setWeight(50)
        self.page.setFont(font)
        self.page.setObjectName("page")
        self.verticalLayout_6 = QtWidgets.QVBoxLayout(self.page)
        self.verticalLayout_6.setObjectName("verticalLayout_6")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout()
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.label = QtWidgets.QLabel(self.page)
        self.label.setObjectName("label")
        self.horizontalLayout.addWidget(self.label)
        self.comboBoxDataColumn = QtWidgets.QComboBox(self.page)
        self.comboBoxDataColumn.setObjectName("comboBoxDataColumn")
        self.horizontalLayout.addWidget(self.comboBoxDataColumn)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.verticalLayout_3.addLayout(self.horizontalLayout)
        self.label_2 = QtWidgets.QLabel(self.page)
        self.label_2.setObjectName("label_2")
        self.verticalLayout_3.addWidget(self.label_2)
        self.comboBoxStimulusMapping = QtWidgets.QComboBox(self.page)
        self.comboBoxStimulusMapping.setObjectName("comboBoxStimulusMapping")
        self.verticalLayout_3.addWidget(self.comboBoxStimulusMapping)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.label_3 = QtWidgets.QLabel(self.page)
        self.label_3.setObjectName("label_3")
        self.horizontalLayout_2.addWidget(self.label_3)
        self.comboBoxMethod = QtWidgets.QComboBox(self.page)
        self.comboBoxMethod.setObjectName("comboBoxMethod")
        self.horizontalLayout_2.addWidget(self.comboBoxMethod)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem1)
        self.verticalLayout_3.addLayout(self.horizontalLayout_2)
        self.groupBox = QtWidgets.QGroupBox(self.page)
        self.groupBox.setToolTip("")
        self.groupBox.setObjectName("groupBox")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.groupBox)
        self.verticalLayout.setObjectName("verticalLayout")
        self.plainTextEditMethodParams = QtWidgets.QPlainTextEdit(self.groupBox)
        font = QtGui.QFont()
        font.setBold(False)
        font.setWeight(50)
        self.plainTextEditMethodParams.setFont(font)
        self.plainTextEditMethodParams.setObjectName("plainTextEditMethodParams")
        self.verticalLayout.addWidget(self.plainTextEditMethodParams)
        self.verticalLayout_3.addWidget(self.groupBox)
        self.checkBoxUsePreprocessScaler = QtWidgets.QCheckBox(self.page)
        self.checkBoxUsePreprocessScaler.setChecked(True)
        self.checkBoxUsePreprocessScaler.setObjectName("checkBoxUsePreprocessScaler")
        self.verticalLayout_3.addWidget(self.checkBoxUsePreprocessScaler)
        self.comboBoxPreprocessScaler = QtWidgets.QComboBox(self.page)
        self.comboBoxPreprocessScaler.setObjectName("comboBoxPreprocessScaler")
        self.verticalLayout_3.addWidget(self.comboBoxPreprocessScaler)
        self.label_5 = QtWidgets.QLabel(self.page)
        self.label_5.setObjectName("label_5")
        self.verticalLayout_3.addWidget(self.label_5)
        self.listWidgetStimulusColormap = ColormapListWidget(self.page)
        self.listWidgetStimulusColormap.setObjectName("listWidgetStimulusColormap")
        self.verticalLayout_3.addWidget(self.listWidgetStimulusColormap)
        self.pushButtonComputeAllSamples = QtWidgets.QPushButton(self.page)
        self.pushButtonComputeAllSamples.setObjectName("pushButtonComputeAllSamples")
        self.verticalLayout_3.addWidget(self.pushButtonComputeAllSamples)
        self.verticalLayout_6.addLayout(self.verticalLayout_3)
        self.progressBar = QtWidgets.QProgressBar(self.page)
        self.progressBar.setProperty("value", 0)
        self.progressBar.setObjectName("progressBar")
        self.verticalLayout_6.addWidget(self.progressBar)
        self.toolBox.addItem(self.page, "")
        self.page_2 = QtWidgets.QWidget()
        self.page_2.setGeometry(QtCore.QRect(0, 0, 251, 578))
        self.page_2.setObjectName("page_2")
        self.verticalLayout_5 = QtWidgets.QVBoxLayout(self.page_2)
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.pushButton_3 = QtWidgets.QPushButton(self.page_2)
        self.pushButton_3.setObjectName("pushButton_3")
        self.verticalLayout_5.addWidget(self.pushButton_3)
        self.listWidgetComputedSamples = QtWidgets.QListWidget(self.page_2)
        font = QtGui.QFont()
        font.setBold(False)
        font.setWeight(50)
        self.listWidgetComputedSamples.setFont(font)
        self.listWidgetComputedSamples.setObjectName("listWidgetComputedSamples")
        self.verticalLayout_5.addWidget(self.listWidgetComputedSamples)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.pushButton = QtWidgets.QPushButton(self.page_2)
        self.pushButton.setObjectName("pushButton")
        self.horizontalLayout_3.addWidget(self.pushButton)
        self.verticalLayout_5.addLayout(self.horizontalLayout_3)
        self.toolBox.addItem(self.page_2, "")
        self.verticalLayout_2.addWidget(self.toolBox)
        DockWidget.setWidget(self.dockWidgetContents)

        self.retranslateUi(DockWidget)
        self.toolBox.setCurrentIndex(1)
        QtCore.QMetaObject.connectSlotsByName(DockWidget)
        DockWidget.setTabOrder(self.comboBoxDataColumn, self.comboBoxStimulusMapping)
        DockWidget.setTabOrder(self.comboBoxStimulusMapping, self.comboBoxMethod)
        DockWidget.setTabOrder(self.comboBoxMethod, self.plainTextEditMethodParams)
        DockWidget.setTabOrder(self.plainTextEditMethodParams, self.checkBoxUsePreprocessScaler)
        DockWidget.setTabOrder(self.checkBoxUsePreprocessScaler, self.comboBoxPreprocessScaler)
        DockWidget.setTabOrder(self.comboBoxPreprocessScaler, self.listWidgetStimulusColormap)
        DockWidget.setTabOrder(self.listWidgetStimulusColormap, self.pushButtonComputeAllSamples)
        DockWidget.setTabOrder(self.pushButtonComputeAllSamples, self.pushButton_3)
        DockWidget.setTabOrder(self.pushButton_3, self.listWidgetComputedSamples)
        DockWidget.setTabOrder(self.listWidgetComputedSamples, self.pushButton)

    def retranslateUi(self, DockWidget):
        _translate = QtCore.QCoreApplication.translate
        DockWidget.setWindowTitle(_translate("DockWidget", "Co&ntrols"))
        self.label.setText(_translate("DockWidget", "Data Column:"))
        self.label_2.setText(_translate("DockWidget", "Stimulus/Behavior mapping:"))
        self.label_3.setText(_translate("DockWidget", "Method:"))
        self.groupBox.setTitle(_translate("DockWidget", "Params for method"))
        self.plainTextEditMethodParams.setPlainText(_translate("DockWidget", "n_components=2"))
        self.checkBoxUsePreprocessScaler.setText(_translate("DockWidget", "Use preprocess scaler:"))
        self.label_5.setText(_translate("DockWidget", "Stimulus colormap:"))
        self.pushButtonComputeAllSamples.setText(_translate("DockWidget", "Compute for all samples"))
        self.toolBox.setItemText(self.toolBox.indexOf(self.page), _translate("DockWidget", "Parameters"))
        self.pushButton_3.setText(_translate("DockWidget", "Open Selected Sample In Viewer"))
        self.pushButton.setText(_translate("DockWidget", "Save"))
        self.toolBox.setItemText(self.toolBox.indexOf(self.page_2), _translate("DockWidget", "Computed Samples"))

from ...utils import ColormapListWidget
