# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file './control_widget.ui'
#
# Created by: PyQt5 UI code generator 5.9.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_Controls(object):
    def setupUi(self, Controls):
        Controls.setObjectName("Controls")
        Controls.resize(325, 851)
        self.dockWidgetContents = QtWidgets.QWidget()
        self.dockWidgetContents.setObjectName("dockWidgetContents")
        self.horizontalLayout_6 = QtWidgets.QHBoxLayout(self.dockWidgetContents)
        self.horizontalLayout_6.setObjectName("horizontalLayout_6")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout()
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.label = QtWidgets.QLabel(self.dockWidgetContents)
        self.label.setObjectName("label")
        self.horizontalLayout.addWidget(self.label)
        self.combo_categorical_column = ComboBox(self.dockWidgetContents)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.combo_categorical_column.sizePolicy().hasHeightForWidth())
        self.combo_categorical_column.setSizePolicy(sizePolicy)
        self.combo_categorical_column.setObjectName("combo_categorical_column")
        self.horizontalLayout.addWidget(self.combo_categorical_column)
        self.verticalLayout_2.addLayout(self.horizontalLayout)
        self.label_2 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_2.setObjectName("label_2")
        self.verticalLayout_2.addWidget(self.label_2)
        self.cmap_img = ColormapListWidget(self.dockWidgetContents)
        self.cmap_img.setMaximumSize(QtCore.QSize(16777215, 150))
        self.cmap_img.setObjectName("cmap_img")
        self.verticalLayout_2.addWidget(self.cmap_img)
        self.label_3 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_3.setObjectName("label_3")
        self.verticalLayout_2.addWidget(self.label_3)
        self.cmap_patches = ColormapListWidget(self.dockWidgetContents)
        self.cmap_patches.setMaximumSize(QtCore.QSize(16777215, 150))
        self.cmap_patches.setObjectName("cmap_patches")
        self.verticalLayout_2.addWidget(self.cmap_patches)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.label_5 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_5.setObjectName("label_5")
        self.horizontalLayout_2.addWidget(self.label_5)
        self.radioButtonMax = QtWidgets.QRadioButton(self.dockWidgetContents)
        self.radioButtonMax.setObjectName("radioButtonMax")
        self.horizontalLayout_2.addWidget(self.radioButtonMax)
        self.radioButtonStd = QtWidgets.QRadioButton(self.dockWidgetContents)
        self.radioButtonStd.setChecked(True)
        self.radioButtonStd.setObjectName("radioButtonStd")
        self.horizontalLayout_2.addWidget(self.radioButtonStd)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem)
        self.verticalLayout_2.addLayout(self.horizontalLayout_2)
        self.checkBoxFill = QtWidgets.QCheckBox(self.dockWidgetContents)
        self.checkBoxFill.setObjectName("checkBoxFill")
        self.verticalLayout_2.addWidget(self.checkBoxFill)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.label_6 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_6.setObjectName("label_6")
        self.horizontalLayout_3.addWidget(self.label_6)
        self.doubleSpinBoxLineWidth = QtWidgets.QDoubleSpinBox(self.dockWidgetContents)
        self.doubleSpinBoxLineWidth.setMaximum(50.0)
        self.doubleSpinBoxLineWidth.setSingleStep(0.25)
        self.doubleSpinBoxLineWidth.setProperty("value", 0.5)
        self.doubleSpinBoxLineWidth.setObjectName("doubleSpinBoxLineWidth")
        self.horizontalLayout_3.addWidget(self.doubleSpinBoxLineWidth)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem1)
        self.verticalLayout_2.addLayout(self.horizontalLayout_3)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.label_7 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_7.setObjectName("label_7")
        self.horizontalLayout_4.addWidget(self.label_7)
        self.doubleSpinBoxAlpha = QtWidgets.QDoubleSpinBox(self.dockWidgetContents)
        self.doubleSpinBoxAlpha.setMaximum(1.0)
        self.doubleSpinBoxAlpha.setSingleStep(0.05)
        self.doubleSpinBoxAlpha.setProperty("value", 0.7)
        self.doubleSpinBoxAlpha.setObjectName("doubleSpinBoxAlpha")
        self.horizontalLayout_4.addWidget(self.doubleSpinBoxAlpha)
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_4.addItem(spacerItem2)
        self.verticalLayout_2.addLayout(self.horizontalLayout_4)
        self.label_4 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_4.setObjectName("label_4")
        self.verticalLayout_2.addWidget(self.label_4)
        self.list_widget_samples = ListWidget(self.dockWidgetContents)
        self.list_widget_samples.setObjectName("list_widget_samples")
        self.verticalLayout_2.addWidget(self.list_widget_samples)
        self.checkBoxLiveUpdate = QtWidgets.QCheckBox(self.dockWidgetContents)
        self.checkBoxLiveUpdate.setObjectName("checkBoxLiveUpdate")
        self.verticalLayout_2.addWidget(self.checkBoxLiveUpdate)
        self.pushButtonUpdatePlot = QtWidgets.QPushButton(self.dockWidgetContents)
        self.pushButtonUpdatePlot.setObjectName("pushButtonUpdatePlot")
        self.verticalLayout_2.addWidget(self.pushButtonUpdatePlot)
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.pushButtonSave = QtWidgets.QPushButton(self.dockWidgetContents)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.pushButtonSave.setFont(font)
        self.pushButtonSave.setObjectName("pushButtonSave")
        self.horizontalLayout_5.addWidget(self.pushButtonSave)
        self.pushButtonLoad = QtWidgets.QPushButton(self.dockWidgetContents)
        self.pushButtonLoad.setObjectName("pushButtonLoad")
        self.horizontalLayout_5.addWidget(self.pushButtonLoad)
        self.verticalLayout_2.addLayout(self.horizontalLayout_5)
        self.horizontalLayout_6.addLayout(self.verticalLayout_2)
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.groupBox = QtWidgets.QGroupBox(self.dockWidgetContents)
        self.groupBox.setObjectName("groupBox")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.groupBox)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.spinBoxZLevel = QtWidgets.QSpinBox(self.groupBox)
        self.spinBoxZLevel.setObjectName("spinBoxZLevel")
        self.verticalLayout_3.addWidget(self.spinBoxZLevel)
        self.verticalSliderZLevel = QtWidgets.QSlider(self.groupBox)
        self.verticalSliderZLevel.setPageStep(5)
        self.verticalSliderZLevel.setOrientation(QtCore.Qt.Vertical)
        self.verticalSliderZLevel.setInvertedAppearance(True)
        self.verticalSliderZLevel.setObjectName("verticalSliderZLevel")
        self.verticalLayout_3.addWidget(self.verticalSliderZLevel)
        self.verticalLayout.addWidget(self.groupBox)
        self.horizontalLayout_6.addLayout(self.verticalLayout)
        Controls.setWidget(self.dockWidgetContents)

        self.retranslateUi(Controls)
        self.verticalSliderZLevel.valueChanged['int'].connect(self.spinBoxZLevel.setValue)
        self.spinBoxZLevel.valueChanged['int'].connect(self.verticalSliderZLevel.setValue)
        QtCore.QMetaObject.connectSlotsByName(Controls)

    def retranslateUi(self, Controls):
        _translate = QtCore.QCoreApplication.translate
        Controls.setWindowTitle(_translate("Controls", "Co&ntrols"))
        self.label.setText(_translate("Controls", "Patch Labels"))
        self.label_2.setText(_translate("Controls", "Image Colormap"))
        self.label_3.setText(_translate("Controls", "Patches Colormap"))
        self.label_5.setText(_translate("Controls", "Projection"))
        self.radioButtonMax.setText(_translate("Controls", "&Max"))
        self.radioButtonStd.setText(_translate("Controls", "Std"))
        self.checkBoxFill.setText(_translate("Controls", "Fill Patches"))
        self.label_6.setText(_translate("Controls", "Line width"))
        self.label_7.setText(_translate("Controls", "Alpha"))
        self.label_4.setText(_translate("Controls", "Samples"))
        self.checkBoxLiveUpdate.setText(_translate("Controls", "Live update from flowchart"))
        self.pushButtonUpdatePlot.setText(_translate("Controls", "Update Plot"))
        self.pushButtonSave.setText(_translate("Controls", "Save"))
        self.pushButtonLoad.setText(_translate("Controls", "Load"))
        self.groupBox.setTitle(_translate("Controls", "z-level"))

from ....pyqtgraphCore.widgets.ComboBox import ComboBox
from ....pyqtgraphCore.widgets.ListWidget import ListWidget
from ...utils import ColormapListWidget
