# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'stats_gui_pytemplate.ui'
#
# Created by: PyQt5 UI code generator 5.9.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(1042, 709)
        self.centralwidget = QtWidgets.QWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.gridLayout = QtWidgets.QGridLayout(self.centralwidget)
        self.gridLayout.setObjectName("gridLayout")
        self.tabWidget = QtWidgets.QTabWidget(self.centralwidget)
        self.tabWidget.setEnabled(True)
        self.tabWidget.setTabBarAutoHide(True)
        self.tabWidget.setObjectName("tabWidget")
        self.groups_tab = QtWidgets.QWidget()
        self.groups_tab.setObjectName("groups_tab")
        self.gridLayout_4 = QtWidgets.QGridLayout(self.groups_tab)
        self.gridLayout_4.setObjectName("gridLayout_4")
        self.label = QtWidgets.QLabel(self.groups_tab)
        self.label.setObjectName("label")
        self.gridLayout_4.addWidget(self.label, 0, 0, 1, 1)
        self.label_2 = QtWidgets.QLabel(self.groups_tab)
        self.label_2.setObjectName("label_2")
        self.gridLayout_4.addWidget(self.label_2, 0, 1, 1, 1)
        self.listwGroups = QtWidgets.QListWidget(self.groups_tab)
        self.listwGroups.setDragDropMode(QtWidgets.QAbstractItemView.DragDrop)
        self.listwGroups.setObjectName("listwGroups")
        self.gridLayout_4.addWidget(self.listwGroups, 1, 0, 1, 1)
        self.listwStats = QtWidgets.QListWidget(self.groups_tab)
        self.listwStats.setDragDropMode(QtWidgets.QAbstractItemView.DragDrop)
        self.listwStats.setObjectName("listwStats")
        self.gridLayout_4.addWidget(self.listwStats, 1, 1, 1, 1)
        self.pushButton = QtWidgets.QPushButton(self.groups_tab)
        self.pushButton.setObjectName("pushButton")
        self.gridLayout_4.addWidget(self.pushButton, 0, 2, 1, 1)
        self.tabWidget.addTab(self.groups_tab, "")
        self.peak_plot_tab = QtWidgets.QWidget()
        self.peak_plot_tab.setObjectName("peak_plot_tab")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.peak_plot_tab)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.tabWidget.addTab(self.peak_plot_tab, "")
        self.stim_plots_tab = QtWidgets.QWidget()
        self.stim_plots_tab.setObjectName("stim_plots_tab")
        self.gridLayout_6 = QtWidgets.QGridLayout(self.stim_plots_tab)
        self.gridLayout_6.setObjectName("gridLayout_6")
        self.tabWidget.addTab(self.stim_plots_tab, "")
        self.violin_tab = QtWidgets.QWidget()
        self.violin_tab.setObjectName("violin_tab")
        self.gridLayout_3 = QtWidgets.QGridLayout(self.violin_tab)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.tabWidget.addTab(self.violin_tab, "")
        self.beeswarm_tab = QtWidgets.QWidget()
        self.beeswarm_tab.setObjectName("beeswarm_tab")
        self.gridLayout_8 = QtWidgets.QGridLayout(self.beeswarm_tab)
        self.gridLayout_8.setObjectName("gridLayout_8")
        self.tabWidget.addTab(self.beeswarm_tab, "")
        self.paracor_tab = QtWidgets.QWidget()
        self.paracor_tab.setObjectName("paracor_tab")
        self.gridLayout_7 = QtWidgets.QGridLayout(self.paracor_tab)
        self.gridLayout_7.setObjectName("gridLayout_7")
        self.tabWidget.addTab(self.paracor_tab, "")
        self.gridLayout.addWidget(self.tabWidget, 0, 0, 1, 1)
        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar(MainWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 1042, 23))
        self.menubar.setObjectName("menubar")
        self.menuFile = QtWidgets.QMenu(self.menubar)
        self.menuFile.setObjectName("menuFile")
        self.menuView = QtWidgets.QMenu(self.menubar)
        self.menuView.setObjectName("menuView")
        MainWindow.setMenuBar(self.menubar)
        self.statusbar = QtWidgets.QStatusBar(MainWindow)
        self.statusbar.setObjectName("statusbar")
        MainWindow.setStatusBar(self.statusbar)
        self.dockWidget = QtWidgets.QDockWidget(MainWindow)
        self.dockWidget.setMinimumSize(QtCore.QSize(250, 66))
        self.dockWidget.setFeatures(QtWidgets.QDockWidget.DockWidgetFloatable|QtWidgets.QDockWidget.DockWidgetMovable)
        self.dockWidget.setWindowTitle("")
        self.dockWidget.setObjectName("dockWidget")
        self.dockWidgetContents = QtWidgets.QWidget()
        self.dockWidgetContents.setObjectName("dockWidgetContents")
        self.gridLayout_5 = QtWidgets.QGridLayout(self.dockWidgetContents)
        self.gridLayout_5.setObjectName("gridLayout_5")
        self.stackedWidget = QtWidgets.QStackedWidget(self.dockWidgetContents)
        self.stackedWidget.setObjectName("stackedWidget")
        self.stack_page_transmission_history = QtWidgets.QWidget()
        self.stack_page_transmission_history.setObjectName("stack_page_transmission_history")
        self.stackedWidget.addWidget(self.stack_page_transmission_history)
        self.stack_page_curve_plots = QtWidgets.QWidget()
        self.stack_page_curve_plots.setObjectName("stack_page_curve_plots")
        self.stackedWidget.addWidget(self.stack_page_curve_plots)
        self.stack_page_stim_plots = QtWidgets.QWidget()
        self.stack_page_stim_plots.setObjectName("stack_page_stim_plots")
        self.stackedWidget.addWidget(self.stack_page_stim_plots)
        self.stack_page_violin_plots = QtWidgets.QWidget()
        self.stack_page_violin_plots.setObjectName("stack_page_violin_plots")
        self.stackedWidget.addWidget(self.stack_page_violin_plots)
        self.stack_page_beeswarm_plots = QtWidgets.QWidget()
        self.stack_page_beeswarm_plots.setObjectName("stack_page_beeswarm_plots")
        self.stackedWidget.addWidget(self.stack_page_beeswarm_plots)
        self.stack_page_paracor_plots = QtWidgets.QWidget()
        self.stack_page_paracor_plots.setObjectName("stack_page_paracor_plots")
        self.stackedWidget.addWidget(self.stack_page_paracor_plots)
        self.gridLayout_5.addWidget(self.stackedWidget, 0, 0, 1, 1)
        self.dockWidget.setWidget(self.dockWidgetContents)
        MainWindow.addDockWidget(QtCore.Qt.DockWidgetArea(1), self.dockWidget)
        self.dockConsole = QtWidgets.QDockWidget(MainWindow)
        self.dockConsole.setMinimumSize(QtCore.QSize(59, 200))
        self.dockConsole.setFeatures(QtWidgets.QDockWidget.DockWidgetFloatable|QtWidgets.QDockWidget.DockWidgetMovable)
        self.dockConsole.setObjectName("dockConsole")
        self.dockWidgetContents_2 = QtWidgets.QWidget()
        self.dockWidgetContents_2.setObjectName("dockWidgetContents_2")
        self.dockConsole.setWidget(self.dockWidgetContents_2)
        MainWindow.addDockWidget(QtCore.Qt.DockWidgetArea(8), self.dockConsole)
        self.actionSave_Statistics_DataFrame = QtWidgets.QAction(MainWindow)
        self.actionSave_Statistics_DataFrame.setObjectName("actionSave_Statistics_DataFrame")
        self.actionSave_Group_Transmissions = QtWidgets.QAction(MainWindow)
        self.actionSave_Group_Transmissions.setObjectName("actionSave_Group_Transmissions")
        self.actionLoad_Groups = QtWidgets.QAction(MainWindow)
        self.actionLoad_Groups.setObjectName("actionLoad_Groups")
        self.actionConsole = QtWidgets.QAction(MainWindow)
        self.actionConsole.setCheckable(True)
        self.actionConsole.setObjectName("actionConsole")
        self.actionSave_Incoming_Transmissions = QtWidgets.QAction(MainWindow)
        self.actionSave_Incoming_Transmissions.setObjectName("actionSave_Incoming_Transmissions")
        self.actionOpen_Statistics_DataFrame = QtWidgets.QAction(MainWindow)
        self.actionOpen_Statistics_DataFrame.setObjectName("actionOpen_Statistics_DataFrame")
        self.actionControls_Dock = QtWidgets.QAction(MainWindow)
        self.actionControls_Dock.setCheckable(True)
        self.actionControls_Dock.setChecked(True)
        self.actionControls_Dock.setObjectName("actionControls_Dock")
        self.actionExport_Statistics_DataFrame = QtWidgets.QAction(MainWindow)
        self.actionExport_Statistics_DataFrame.setObjectName("actionExport_Statistics_DataFrame")
        self.actionExport_Groups = QtWidgets.QAction(MainWindow)
        self.actionExport_Groups.setObjectName("actionExport_Groups")
        self.menuFile.addAction(self.actionSave_Statistics_DataFrame)
        self.menuFile.addAction(self.actionOpen_Statistics_DataFrame)
        self.menuFile.addAction(self.actionExport_Statistics_DataFrame)
        self.menuFile.addSeparator()
        self.menuFile.addAction(self.actionSave_Group_Transmissions)
        self.menuFile.addAction(self.actionLoad_Groups)
        self.menuFile.addAction(self.actionExport_Groups)
        self.menuFile.addSeparator()
        self.menuFile.addAction(self.actionSave_Incoming_Transmissions)
        self.menuView.addAction(self.actionConsole)
        self.menuView.addAction(self.actionControls_Dock)
        self.menubar.addAction(self.menuFile.menuAction())
        self.menubar.addAction(self.menuView.menuAction())

        self.retranslateUi(MainWindow)
        self.tabWidget.setCurrentIndex(4)
        self.stackedWidget.setCurrentIndex(5)
        self.actionConsole.toggled['bool'].connect(self.dockConsole.setVisible)
        self.actionControls_Dock.toggled['bool'].connect(self.dockWidget.setVisible)
        self.tabWidget.currentChanged['int'].connect(self.stackedWidget.setCurrentIndex)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("MainWindow", "MainWindow"))
        self.label.setText(_translate("MainWindow", "Open Groups"))
        self.label_2.setText(_translate("MainWindow", "Groups for Stats & Plots"))
        self.pushButton.setText(_translate("MainWindow", "Set"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.groups_tab), _translate("MainWindow", "Groups"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.peak_plot_tab), _translate("MainWindow", "Peak Plots"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.stim_plots_tab), _translate("MainWindow", "Stim Plots"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.violin_tab), _translate("MainWindow", "Violin Plots"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.beeswarm_tab), _translate("MainWindow", "Beeswarm Plots"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.paracor_tab), _translate("MainWindow", "Parallel Coordinates"))
        self.menuFile.setTitle(_translate("MainWindow", "File"))
        self.menuView.setTitle(_translate("MainWindow", "View"))
        self.dockConsole.setWindowTitle(_translate("MainWindow", "Statistics Console"))
        self.actionSave_Statistics_DataFrame.setText(_translate("MainWindow", "Save Statistics DataFrame"))
        self.actionSave_Group_Transmissions.setText(_translate("MainWindow", "Save Groups"))
        self.actionLoad_Groups.setText(_translate("MainWindow", "Load Groups"))
        self.actionConsole.setText(_translate("MainWindow", "Console"))
        self.actionSave_Incoming_Transmissions.setText(_translate("MainWindow", "Save Incoming Raw Transmissions"))
        self.actionOpen_Statistics_DataFrame.setText(_translate("MainWindow", "Open Statistics DataFrame"))
        self.actionControls_Dock.setText(_translate("MainWindow", "Controls Dock"))
        self.actionExport_Statistics_DataFrame.setText(_translate("MainWindow", "Export Statistics DataFrame"))
        self.actionExport_Groups.setText(_translate("MainWindow", "Export Groups"))

