# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file './template.ui'
#
# Created by: PyQt5 UI code generator 5.9.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(1146, 661)
        self.centralwidget = QtWidgets.QWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.verticalLayout_5 = QtWidgets.QVBoxLayout(self.centralwidget)
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem)
        self.label = QtWidgets.QLabel(self.centralwidget)
        self.label.setMinimumSize(QtCore.QSize(0, 0))
        font = QtGui.QFont()
        font.setPointSize(16)
        font.setBold(True)
        font.setWeight(75)
        self.label.setFont(font)
        self.label.setObjectName("label")
        self.horizontalLayout_3.addWidget(self.label)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem1)
        self.verticalLayout.addLayout(self.horizontalLayout_3)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.btnModeView = QtWidgets.QPushButton(self.centralwidget)
        self.btnModeView.setMinimumSize(QtCore.QSize(0, 36))
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.btnModeView.setFont(font)
        self.btnModeView.setCheckable(True)
        self.btnModeView.setObjectName("btnModeView")
        self.horizontalLayout.addWidget(self.btnModeView)
        self.btnModeDrag = QtWidgets.QPushButton(self.centralwidget)
        self.btnModeDrag.setMinimumSize(QtCore.QSize(0, 36))
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.btnModeDrag.setFont(font)
        self.btnModeDrag.setCheckable(True)
        self.btnModeDrag.setObjectName("btnModeDrag")
        self.horizontalLayout.addWidget(self.btnModeDrag)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.btnModeAddPeaks = QtWidgets.QPushButton(self.centralwidget)
        self.btnModeAddPeaks.setMinimumSize(QtCore.QSize(0, 36))
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.btnModeAddPeaks.setFont(font)
        self.btnModeAddPeaks.setCheckable(True)
        self.btnModeAddPeaks.setObjectName("btnModeAddPeaks")
        self.horizontalLayout_2.addWidget(self.btnModeAddPeaks)
        self.btnModeAddBases = QtWidgets.QPushButton(self.centralwidget)
        self.btnModeAddBases.setMinimumSize(QtCore.QSize(0, 36))
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.btnModeAddBases.setFont(font)
        self.btnModeAddBases.setCheckable(True)
        self.btnModeAddBases.setObjectName("btnModeAddBases")
        self.horizontalLayout_2.addWidget(self.btnModeAddBases)
        self.btnModeDelete = QtWidgets.QPushButton(self.centralwidget)
        self.btnModeDelete.setMinimumSize(QtCore.QSize(0, 36))
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.btnModeDelete.setFont(font)
        self.btnModeDelete.setCheckable(True)
        self.btnModeDelete.setObjectName("btnModeDelete")
        self.horizontalLayout_2.addWidget(self.btnModeDelete)
        self.verticalLayout.addLayout(self.horizontalLayout_2)
        self.horizontalLayout_4.addLayout(self.verticalLayout)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout()
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setObjectName("gridLayout")
        self.label_3 = QtWidgets.QLabel(self.centralwidget)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_3.setFont(font)
        self.label_3.setObjectName("label_3")
        self.gridLayout.addWidget(self.label_3, 0, 0, 1, 1)
        self.label_4 = QtWidgets.QLabel(self.centralwidget)
        self.label_4.setObjectName("label_4")
        self.gridLayout.addWidget(self.label_4, 0, 1, 1, 1)
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout.addItem(spacerItem2, 0, 2, 1, 1)
        self.sliderDotSize = QtWidgets.QSlider(self.centralwidget)
        self.sliderDotSize.setMinimum(5)
        self.sliderDotSize.setMaximum(50)
        self.sliderDotSize.setSingleStep(5)
        self.sliderDotSize.setPageStep(5)
        self.sliderDotSize.setProperty("value", 15)
        self.sliderDotSize.setOrientation(QtCore.Qt.Horizontal)
        self.sliderDotSize.setObjectName("sliderDotSize")
        self.gridLayout.addWidget(self.sliderDotSize, 1, 0, 1, 3)
        self.verticalLayout_2.addLayout(self.gridLayout)
        self.btnSetPeaksAtRelativeMaxima = QtWidgets.QPushButton(self.centralwidget)
        self.btnSetPeaksAtRelativeMaxima.setMinimumSize(QtCore.QSize(125, 55))
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.btnSetPeaksAtRelativeMaxima.setFont(font)
        self.btnSetPeaksAtRelativeMaxima.setObjectName("btnSetPeaksAtRelativeMaxima")
        self.verticalLayout_2.addWidget(self.btnSetPeaksAtRelativeMaxima)
        self.horizontalLayout_4.addLayout(self.verticalLayout_2)
        self.verticalLayout_4 = QtWidgets.QVBoxLayout()
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.btnSaveCurve = QtWidgets.QPushButton(self.centralwidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.MinimumExpanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.btnSaveCurve.sizePolicy().hasHeightForWidth())
        self.btnSaveCurve.setSizePolicy(sizePolicy)
        self.btnSaveCurve.setMinimumSize(QtCore.QSize(96, 72))
        self.btnSaveCurve.setMaximumSize(QtCore.QSize(360, 73))
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.btnSaveCurve.setFont(font)
        self.btnSaveCurve.setObjectName("btnSaveCurve")
        self.verticalLayout_4.addWidget(self.btnSaveCurve)
        self.btnClearCurve = QtWidgets.QPushButton(self.centralwidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.MinimumExpanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.btnClearCurve.sizePolicy().hasHeightForWidth())
        self.btnClearCurve.setSizePolicy(sizePolicy)
        self.btnClearCurve.setMinimumSize(QtCore.QSize(96, 72))
        self.btnClearCurve.setMaximumSize(QtCore.QSize(360, 73))
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.btnClearCurve.setFont(font)
        self.btnClearCurve.setObjectName("btnClearCurve")
        self.verticalLayout_4.addWidget(self.btnClearCurve)
        self.horizontalLayout_4.addLayout(self.verticalLayout_4)
        self.verticalLayout_3 = QtWidgets.QVBoxLayout()
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.horizontalLayout_6 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_6.setObjectName("horizontalLayout_6")
        self.btnDisconnectFromFlowchart = QtWidgets.QPushButton(self.centralwidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.MinimumExpanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.btnDisconnectFromFlowchart.sizePolicy().hasHeightForWidth())
        self.btnDisconnectFromFlowchart.setSizePolicy(sizePolicy)
        self.btnDisconnectFromFlowchart.setMinimumSize(QtCore.QSize(220, 30))
        self.btnDisconnectFromFlowchart.setMaximumSize(QtCore.QSize(340, 80))
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.btnDisconnectFromFlowchart.setFont(font)
        self.btnDisconnectFromFlowchart.setCheckable(True)
        self.btnDisconnectFromFlowchart.setObjectName("btnDisconnectFromFlowchart")
        self.horizontalLayout_6.addWidget(self.btnDisconnectFromFlowchart)
        self.btnSendToFlowchart = QtWidgets.QPushButton(self.centralwidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.MinimumExpanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.btnSendToFlowchart.sizePolicy().hasHeightForWidth())
        self.btnSendToFlowchart.setSizePolicy(sizePolicy)
        self.btnSendToFlowchart.setMinimumSize(QtCore.QSize(220, 30))
        self.btnSendToFlowchart.setMaximumSize(QtCore.QSize(340, 80))
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.btnSendToFlowchart.setFont(font)
        self.btnSendToFlowchart.setObjectName("btnSendToFlowchart")
        self.horizontalLayout_6.addWidget(self.btnSendToFlowchart)
        self.verticalLayout_3.addLayout(self.horizontalLayout_6)
        self.pushButtonOpenInViewer = QtWidgets.QPushButton(self.centralwidget)
        self.pushButtonOpenInViewer.setObjectName("pushButtonOpenInViewer")
        self.verticalLayout_3.addWidget(self.pushButtonOpenInViewer)
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.label_2 = QtWidgets.QLabel(self.centralwidget)
        font = QtGui.QFont()
        font.setPointSize(12)
        font.setBold(True)
        font.setWeight(75)
        self.label_2.setFont(font)
        self.label_2.setObjectName("label_2")
        self.horizontalLayout_5.addWidget(self.label_2)
        self.lineEditSampleID = QtWidgets.QLineEdit(self.centralwidget)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.lineEditSampleID.setFont(font)
        self.lineEditSampleID.setReadOnly(True)
        self.lineEditSampleID.setObjectName("lineEditSampleID")
        self.horizontalLayout_5.addWidget(self.lineEditSampleID)
        self.verticalLayout_3.addLayout(self.horizontalLayout_5)
        self.horizontalLayout_4.addLayout(self.verticalLayout_3)
        self.verticalLayout_5.addLayout(self.horizontalLayout_4)
        self.graphicsView = PlotWidget(self.centralwidget)
        self.graphicsView.setObjectName("graphicsView")
        self.verticalLayout_5.addWidget(self.graphicsView)
        self.listwIndices = QtWidgets.QListWidget(self.centralwidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.listwIndices.sizePolicy().hasHeightForWidth())
        self.listwIndices.setSizePolicy(sizePolicy)
        self.listwIndices.setMinimumSize(QtCore.QSize(0, 33))
        self.listwIndices.setMaximumSize(QtCore.QSize(16777215, 33))
        self.listwIndices.setProperty("showDropIndicator", False)
        self.listwIndices.setDefaultDropAction(QtCore.Qt.IgnoreAction)
        self.listwIndices.setSelectionMode(QtWidgets.QAbstractItemView.SingleSelection)
        self.listwIndices.setFlow(QtWidgets.QListView.LeftToRight)
        self.listwIndices.setObjectName("listwIndices")
        self.verticalLayout_5.addWidget(self.listwIndices)
        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar(MainWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 1146, 22))
        self.menubar.setObjectName("menubar")
        self.menuView = QtWidgets.QMenu(self.menubar)
        self.menuView.setObjectName("menuView")
        MainWindow.setMenuBar(self.menubar)
        self.statusbar = QtWidgets.QStatusBar(MainWindow)
        self.statusbar.setObjectName("statusbar")
        MainWindow.setStatusBar(self.statusbar)
        self.dockWidget = QtWidgets.QDockWidget(MainWindow)
        self.dockWidget.setMinimumSize(QtCore.QSize(200, 50))
        self.dockWidget.setObjectName("dockWidget")
        self.dockWidgetContents = QtWidgets.QWidget()
        self.dockWidgetContents.setObjectName("dockWidgetContents")
        self.dockWidget.setWidget(self.dockWidgetContents)
        MainWindow.addDockWidget(QtCore.Qt.DockWidgetArea(2), self.dockWidget)
        self.actionUndo = QtWidgets.QAction(MainWindow)
        self.actionUndo.setObjectName("actionUndo")
        self.actionRedo = QtWidgets.QAction(MainWindow)
        self.actionRedo.setObjectName("actionRedo")
        self.actionTransmissions_dock = QtWidgets.QAction(MainWindow)
        self.actionTransmissions_dock.setCheckable(True)
        self.actionTransmissions_dock.setChecked(True)
        self.actionTransmissions_dock.setObjectName("actionTransmissions_dock")
        self.menuView.addAction(self.actionTransmissions_dock)
        self.menubar.addAction(self.menuView.menuAction())

        self.retranslateUi(MainWindow)
        self.sliderDotSize.valueChanged['int'].connect(self.label_4.setNum)
        self.actionTransmissions_dock.toggled['bool'].connect(self.dockWidget.setVisible)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)
        MainWindow.setTabOrder(self.btnModeView, self.btnModeDrag)
        MainWindow.setTabOrder(self.btnModeDrag, self.btnModeAddPeaks)
        MainWindow.setTabOrder(self.btnModeAddPeaks, self.btnModeAddBases)
        MainWindow.setTabOrder(self.btnModeAddBases, self.btnModeDelete)
        MainWindow.setTabOrder(self.btnModeDelete, self.sliderDotSize)
        MainWindow.setTabOrder(self.sliderDotSize, self.btnSetPeaksAtRelativeMaxima)
        MainWindow.setTabOrder(self.btnSetPeaksAtRelativeMaxima, self.btnSaveCurve)
        MainWindow.setTabOrder(self.btnSaveCurve, self.btnDisconnectFromFlowchart)
        MainWindow.setTabOrder(self.btnDisconnectFromFlowchart, self.lineEditSampleID)
        MainWindow.setTabOrder(self.lineEditSampleID, self.graphicsView)
        MainWindow.setTabOrder(self.graphicsView, self.listwIndices)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("MainWindow", "MainWindow"))
        self.label.setText(_translate("MainWindow", "Mode"))
        self.btnModeView.setText(_translate("MainWindow", "View"))
        self.btnModeDrag.setText(_translate("MainWindow", "Drag"))
        self.btnModeAddPeaks.setText(_translate("MainWindow", "Add Peaks"))
        self.btnModeAddBases.setText(_translate("MainWindow", "Add bases"))
        self.btnModeDelete.setText(_translate("MainWindow", "Delete"))
        self.label_3.setText(_translate("MainWindow", "Dot Size:"))
        self.label_4.setText(_translate("MainWindow", "15"))
        self.btnSetPeaksAtRelativeMaxima.setText(_translate("MainWindow", "Set peaks at\n"
"relative maxima"))
        self.btnSaveCurve.setText(_translate("MainWindow", "Save Curve"))
        self.btnClearCurve.setText(_translate("MainWindow", "Clear Curve"))
        self.btnDisconnectFromFlowchart.setText(_translate("MainWindow", "Disconnect from flowchart"))
        self.btnSendToFlowchart.setText(_translate("MainWindow", "Send all curves to flowchart"))
        self.pushButtonOpenInViewer.setText(_translate("MainWindow", "Open in Viewer"))
        self.label_2.setText(_translate("MainWindow", "SampleID:"))
        self.menuView.setTitle(_translate("MainWindow", "&View"))
        self.actionUndo.setText(_translate("MainWindow", "&Undo"))
        self.actionRedo.setText(_translate("MainWindow", "&Redo"))
        self.actionTransmissions_dock.setText(_translate("MainWindow", "&Transmissions dock"))

from ....pyqtgraphCore import PlotWidget
