# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file './caiman_motion_correction.ui'
#
# Created by: PyQt5 UI code generator 5.12
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_DockWidget(object):
    def setupUi(self, DockWidget):
        DockWidget.setObjectName("DockWidget")
        DockWidget.resize(287, 842)
        DockWidget.setFloating(True)
        self.dockWidgetContents = QtWidgets.QWidget()
        self.dockWidgetContents.setObjectName("dockWidgetContents")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.dockWidgetContents)
        self.verticalLayout.setObjectName("verticalLayout")
        self.groupBox_motion_correction_kwargs = QtWidgets.QGroupBox(self.dockWidgetContents)
        self.groupBox_motion_correction_kwargs.setStatusTip("")
        self.groupBox_motion_correction_kwargs.setCheckable(True)
        self.groupBox_motion_correction_kwargs.setChecked(False)
        self.groupBox_motion_correction_kwargs.setObjectName("groupBox_motion_correction_kwargs")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.groupBox_motion_correction_kwargs)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.plainTextEdit_mc_kwargs = QtWidgets.QPlainTextEdit(self.groupBox_motion_correction_kwargs)
        self.plainTextEdit_mc_kwargs.setObjectName("plainTextEdit_mc_kwargs")
        self.verticalLayout_3.addWidget(self.plainTextEdit_mc_kwargs)
        self.verticalLayout.addWidget(self.groupBox_motion_correction_kwargs)
        self.label_16 = QtWidgets.QLabel(self.dockWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(12)
        font.setBold(True)
        font.setWeight(75)
        self.label_16.setFont(font)
        self.label_16.setObjectName("label_16")
        self.verticalLayout.addWidget(self.label_16)
        self.horizontalLayout_21 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_21.setObjectName("horizontalLayout_21")
        self.label_11 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_11.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.label_11.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_11.setObjectName("label_11")
        self.horizontalLayout_21.addWidget(self.label_11)
        self.comboBoxOutputBitDepth = QtWidgets.QComboBox(self.dockWidgetContents)
        self.comboBoxOutputBitDepth.setObjectName("comboBoxOutputBitDepth")
        self.comboBoxOutputBitDepth.addItem("")
        self.comboBoxOutputBitDepth.addItem("")
        self.comboBoxOutputBitDepth.addItem("")
        self.horizontalLayout_21.addWidget(self.comboBoxOutputBitDepth)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_21.addItem(spacerItem)
        self.verticalLayout.addLayout(self.horizontalLayout_21)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.label = QtWidgets.QLabel(self.dockWidgetContents)
        self.label.setObjectName("label")
        self.horizontalLayout.addWidget(self.label)
        self.spinBoxGSig_filt = QtWidgets.QSpinBox(self.dockWidgetContents)
        self.spinBoxGSig_filt.setObjectName("spinBoxGSig_filt")
        self.horizontalLayout.addWidget(self.spinBoxGSig_filt)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem1)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.horizontalLayout_12 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_12.setObjectName("horizontalLayout_12")
        self.label_9 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_9.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.label_9.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_9.setObjectName("label_9")
        self.horizontalLayout_12.addWidget(self.label_9)
        self.spinboxX = QtWidgets.QSpinBox(self.dockWidgetContents)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.spinboxX.sizePolicy().hasHeightForWidth())
        self.spinboxX.setSizePolicy(sizePolicy)
        self.spinboxX.setMaximum(999)
        self.spinboxX.setObjectName("spinboxX")
        self.horizontalLayout_12.addWidget(self.spinboxX)
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_12.addItem(spacerItem2)
        self.verticalLayout.addLayout(self.horizontalLayout_12)
        self.horizontalLayout_13 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_13.setObjectName("horizontalLayout_13")
        self.label_13 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_13.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_13.setObjectName("label_13")
        self.horizontalLayout_13.addWidget(self.label_13)
        self.spinboxY = QtWidgets.QSpinBox(self.dockWidgetContents)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.spinboxY.sizePolicy().hasHeightForWidth())
        self.spinboxY.setSizePolicy(sizePolicy)
        self.spinboxY.setMaximum(999)
        self.spinboxY.setObjectName("spinboxY")
        self.horizontalLayout_13.addWidget(self.spinboxY)
        spacerItem3 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_13.addItem(spacerItem3)
        self.verticalLayout.addLayout(self.horizontalLayout_13)
        self.horizontalLayout_14 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_14.setObjectName("horizontalLayout_14")
        self.label_6 = QtWidgets.QLabel(self.dockWidgetContents)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_6.sizePolicy().hasHeightForWidth())
        self.label_6.setSizePolicy(sizePolicy)
        self.label_6.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_6.setObjectName("label_6")
        self.horizontalLayout_14.addWidget(self.label_6)
        self.spinboxIterRigid = QtWidgets.QSpinBox(self.dockWidgetContents)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.spinboxIterRigid.sizePolicy().hasHeightForWidth())
        self.spinboxIterRigid.setSizePolicy(sizePolicy)
        self.spinboxIterRigid.setMinimum(1)
        self.spinboxIterRigid.setMaximum(5)
        self.spinboxIterRigid.setObjectName("spinboxIterRigid")
        self.horizontalLayout_14.addWidget(self.spinboxIterRigid)
        spacerItem4 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_14.addItem(spacerItem4)
        self.verticalLayout.addLayout(self.horizontalLayout_14)
        self.label_17 = QtWidgets.QLabel(self.dockWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(12)
        font.setBold(True)
        font.setWeight(75)
        self.label_17.setFont(font)
        self.label_17.setObjectName("label_17")
        self.verticalLayout.addWidget(self.label_17)
        self.horizontalLayout_16 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_16.setObjectName("horizontalLayout_16")
        self.label_15 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_15.setAlignment(QtCore.Qt.AlignCenter)
        self.label_15.setObjectName("label_15")
        self.horizontalLayout_16.addWidget(self.label_15)
        self.spinboxMaxDev = QtWidgets.QSpinBox(self.dockWidgetContents)
        self.spinboxMaxDev.setMaximum(999)
        self.spinboxMaxDev.setProperty("value", 3)
        self.spinboxMaxDev.setObjectName("spinboxMaxDev")
        self.horizontalLayout_16.addWidget(self.spinboxMaxDev)
        spacerItem5 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_16.addItem(spacerItem5)
        self.verticalLayout.addLayout(self.horizontalLayout_16)
        self.verticalLayout_5 = QtWidgets.QVBoxLayout()
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.horizontalLayout_17 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_17.setObjectName("horizontalLayout_17")
        self.label_18 = QtWidgets.QLabel(self.dockWidgetContents)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_18.sizePolicy().hasHeightForWidth())
        self.label_18.setSizePolicy(sizePolicy)
        self.label_18.setAlignment(QtCore.Qt.AlignBottom|QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft)
        self.label_18.setObjectName("label_18")
        self.horizontalLayout_17.addWidget(self.label_18)
        self.spinboxStrides = QtWidgets.QSpinBox(self.dockWidgetContents)
        self.spinboxStrides.setMaximum(500)
        self.spinboxStrides.setProperty("value", 0)
        self.spinboxStrides.setObjectName("spinboxStrides")
        self.horizontalLayout_17.addWidget(self.spinboxStrides)
        spacerItem6 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_17.addItem(spacerItem6)
        self.verticalLayout_5.addLayout(self.horizontalLayout_17)
        self.sliderStrides = QtWidgets.QSlider(self.dockWidgetContents)
        self.sliderStrides.setMinimum(1)
        self.sliderStrides.setMaximum(500)
        self.sliderStrides.setOrientation(QtCore.Qt.Horizontal)
        self.sliderStrides.setObjectName("sliderStrides")
        self.verticalLayout_5.addWidget(self.sliderStrides)
        self.verticalLayout.addLayout(self.verticalLayout_5)
        self.verticalLayout_6 = QtWidgets.QVBoxLayout()
        self.verticalLayout_6.setObjectName("verticalLayout_6")
        self.horizontalLayout_18 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_18.setObjectName("horizontalLayout_18")
        self.label_19 = QtWidgets.QLabel(self.dockWidgetContents)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_19.sizePolicy().hasHeightForWidth())
        self.label_19.setSizePolicy(sizePolicy)
        self.label_19.setAlignment(QtCore.Qt.AlignBottom|QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft)
        self.label_19.setObjectName("label_19")
        self.horizontalLayout_18.addWidget(self.label_19)
        self.spinboxOverlaps = QtWidgets.QSpinBox(self.dockWidgetContents)
        self.spinboxOverlaps.setMaximum(500)
        self.spinboxOverlaps.setProperty("value", 0)
        self.spinboxOverlaps.setObjectName("spinboxOverlaps")
        self.horizontalLayout_18.addWidget(self.spinboxOverlaps)
        spacerItem7 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_18.addItem(spacerItem7)
        self.verticalLayout_6.addLayout(self.horizontalLayout_18)
        self.sliderOverlaps = QtWidgets.QSlider(self.dockWidgetContents)
        self.sliderOverlaps.setMinimum(1)
        self.sliderOverlaps.setMaximum(500)
        self.sliderOverlaps.setOrientation(QtCore.Qt.Horizontal)
        self.sliderOverlaps.setObjectName("sliderOverlaps")
        self.verticalLayout_6.addWidget(self.sliderOverlaps)
        self.verticalLayout.addLayout(self.verticalLayout_6)
        self.btnShowQuilt = QtWidgets.QPushButton(self.dockWidgetContents)
        self.btnShowQuilt.setMaximumSize(QtCore.QSize(100, 16777215))
        self.btnShowQuilt.setCheckable(True)
        self.btnShowQuilt.setObjectName("btnShowQuilt")
        self.verticalLayout.addWidget(self.btnShowQuilt)
        self.horizontalLayout_19 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_19.setObjectName("horizontalLayout_19")
        self.label_10 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_10.setAlignment(QtCore.Qt.AlignCenter)
        self.label_10.setObjectName("label_10")
        self.horizontalLayout_19.addWidget(self.label_10)
        self.spinboxUpsample = QtWidgets.QSpinBox(self.dockWidgetContents)
        self.spinboxUpsample.setMinimum(1)
        self.spinboxUpsample.setMaximum(10)
        self.spinboxUpsample.setProperty("value", 4)
        self.spinboxUpsample.setObjectName("spinboxUpsample")
        self.horizontalLayout_19.addWidget(self.spinboxUpsample)
        spacerItem8 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_19.addItem(spacerItem8)
        self.verticalLayout.addLayout(self.horizontalLayout_19)
        self.horizontalLayout_20 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_20.setObjectName("horizontalLayout_20")
        self.lineEditNameElastic = QtWidgets.QLineEdit(self.dockWidgetContents)
        self.lineEditNameElastic.setObjectName("lineEditNameElastic")
        self.horizontalLayout_20.addWidget(self.lineEditNameElastic)
        self.btnAddToBatchElastic = QtWidgets.QPushButton(self.dockWidgetContents)
        self.btnAddToBatchElastic.setObjectName("btnAddToBatchElastic")
        self.horizontalLayout_20.addWidget(self.btnAddToBatchElastic)
        self.verticalLayout.addLayout(self.horizontalLayout_20)
        self.line = QtWidgets.QFrame(self.dockWidgetContents)
        self.line.setFrameShape(QtWidgets.QFrame.HLine)
        self.line.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line.setObjectName("line")
        self.verticalLayout.addWidget(self.line)
        self.label_2 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_2.setObjectName("label_2")
        self.verticalLayout.addWidget(self.label_2)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.comboBoxProjectionsOption = QtWidgets.QComboBox(self.dockWidgetContents)
        self.comboBoxProjectionsOption.setObjectName("comboBoxProjectionsOption")
        self.comboBoxProjectionsOption.addItem("")
        self.comboBoxProjectionsOption.addItem("")
        self.comboBoxProjectionsOption.addItem("")
        self.horizontalLayout_2.addWidget(self.comboBoxProjectionsOption)
        self.pushButtonViewProjections = QtWidgets.QPushButton(self.dockWidgetContents)
        self.pushButtonViewProjections.setObjectName("pushButtonViewProjections")
        self.horizontalLayout_2.addWidget(self.pushButtonViewProjections)
        self.verticalLayout.addLayout(self.horizontalLayout_2)
        spacerItem9 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem9)
        DockWidget.setWidget(self.dockWidgetContents)

        self.retranslateUi(DockWidget)
        self.sliderOverlaps.valueChanged['int'].connect(self.spinboxOverlaps.setValue)
        self.sliderStrides.valueChanged['int'].connect(self.spinboxStrides.setValue)
        self.spinboxOverlaps.valueChanged['int'].connect(self.sliderOverlaps.setValue)
        self.spinboxStrides.valueChanged['int'].connect(self.sliderStrides.setValue)
        self.lineEditNameElastic.returnPressed.connect(self.btnAddToBatchElastic.click)
        QtCore.QMetaObject.connectSlotsByName(DockWidget)
        DockWidget.setTabOrder(self.groupBox_motion_correction_kwargs, self.plainTextEdit_mc_kwargs)
        DockWidget.setTabOrder(self.plainTextEdit_mc_kwargs, self.comboBoxOutputBitDepth)
        DockWidget.setTabOrder(self.comboBoxOutputBitDepth, self.spinBoxGSig_filt)
        DockWidget.setTabOrder(self.spinBoxGSig_filt, self.spinboxX)
        DockWidget.setTabOrder(self.spinboxX, self.spinboxY)
        DockWidget.setTabOrder(self.spinboxY, self.spinboxIterRigid)
        DockWidget.setTabOrder(self.spinboxIterRigid, self.spinboxMaxDev)
        DockWidget.setTabOrder(self.spinboxMaxDev, self.spinboxStrides)
        DockWidget.setTabOrder(self.spinboxStrides, self.sliderStrides)
        DockWidget.setTabOrder(self.sliderStrides, self.spinboxOverlaps)
        DockWidget.setTabOrder(self.spinboxOverlaps, self.sliderOverlaps)
        DockWidget.setTabOrder(self.sliderOverlaps, self.btnShowQuilt)
        DockWidget.setTabOrder(self.btnShowQuilt, self.spinboxUpsample)
        DockWidget.setTabOrder(self.spinboxUpsample, self.lineEditNameElastic)
        DockWidget.setTabOrder(self.lineEditNameElastic, self.btnAddToBatchElastic)

    def retranslateUi(self, DockWidget):
        _translate = QtCore.QCoreApplication.translate
        DockWidget.setWindowTitle(_translate("DockWidget", "&CaImAn Motion Correction"))
        self.groupBox_motion_correction_kwargs.setToolTip(_translate("DockWidget", "You can enter additional kwargs to pass for MotionCorrect() instantiation.\n"
"Use single quotes for strings, do not use double quotes."))
        self.groupBox_motion_correction_kwargs.setTitle(_translate("DockWidget", "Use &motion correction kwargs"))
        self.plainTextEdit_mc_kwargs.setToolTip(_translate("DockWidget", "You can enter additional kwargs to pass for CNMF instantiation.\n"
"Use single quotes for strings, do not use double quotes."))
        self.label_16.setToolTip(_translate("DockWidget", "Compute some summary images (correlation and peak to noise ratio)"))
        self.label_16.setText(_translate("DockWidget", "Rigid correction"))
        self.label_11.setText(_translate("DockWidget", "Output bit depth: "))
        self.comboBoxOutputBitDepth.setItemText(0, _translate("DockWidget", "Do not convert"))
        self.comboBoxOutputBitDepth.setItemText(1, _translate("DockWidget", "8"))
        self.comboBoxOutputBitDepth.setItemText(2, _translate("DockWidget", "16"))
        self.label.setText(_translate("DockWidget", "gSig_filt:"))
        self.spinBoxGSig_filt.setToolTip(_translate("DockWidget", "disabled if set to zero"))
        self.label_9.setText(_translate("DockWidget", " max shifts X (pixels): "))
        self.label_13.setText(_translate("DockWidget", " max shifts Y (pixels): "))
        self.label_6.setText(_translate("DockWidget", " iterations for rigid: "))
        self.label_17.setToolTip(_translate("DockWidget", "Compute some summary images (correlation and peak to noise ratio)"))
        self.label_17.setText(_translate("DockWidget", "Elastic correction"))
        self.label_15.setText(_translate("DockWidget", " max deviation from rigid:"))
        self.label_18.setText(_translate("DockWidget", "strides (pixels): "))
        self.label_19.setText(_translate("DockWidget", "overlaps (pixels): "))
        self.btnShowQuilt.setText(_translate("DockWidget", "Show Grid"))
        self.label_10.setText(_translate("DockWidget", " upsample grid:"))
        self.lineEditNameElastic.setPlaceholderText(_translate("DockWidget", "Enter name"))
        self.btnAddToBatchElastic.setText(_translate("DockWidget", "Add to batch"))
        self.label_2.setToolTip(_translate("DockWidget", "View Mean, Max or Std projections of the outputs from selected batch items"))
        self.label_2.setText(_translate("DockWidget", "View projections of selected outputs"))
        self.comboBoxProjectionsOption.setToolTip(_translate("DockWidget", "View Mean, Max or Std projections of the outputs from selected batch items"))
        self.comboBoxProjectionsOption.setItemText(0, _translate("DockWidget", "mean"))
        self.comboBoxProjectionsOption.setItemText(1, _translate("DockWidget", "max"))
        self.comboBoxProjectionsOption.setItemText(2, _translate("DockWidget", "std"))
        self.pushButtonViewProjections.setToolTip(_translate("DockWidget", "View Mean, Max or Std projections of the outputs from selected batch items"))
        self.pushButtonViewProjections.setText(_translate("DockWidget", "View"))


# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file './caiman_motion_correction.ui'
#
# Created by: PyQt5 UI code generator 5.12
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_DockWidget(object):
    def setupUi(self, DockWidget):
        DockWidget.setObjectName("DockWidget")
        DockWidget.resize(279, 869)
        DockWidget.setFloating(True)
        self.dockWidgetContents = QtWidgets.QWidget()
        self.dockWidgetContents.setObjectName("dockWidgetContents")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.dockWidgetContents)
        self.verticalLayout.setObjectName("verticalLayout")
        self.groupBox_motion_correction_kwargs = QtWidgets.QGroupBox(self.dockWidgetContents)
        self.groupBox_motion_correction_kwargs.setStatusTip("")
        self.groupBox_motion_correction_kwargs.setCheckable(True)
        self.groupBox_motion_correction_kwargs.setChecked(False)
        self.groupBox_motion_correction_kwargs.setObjectName("groupBox_motion_correction_kwargs")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.groupBox_motion_correction_kwargs)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.plainTextEdit_mc_kwargs = QtWidgets.QPlainTextEdit(self.groupBox_motion_correction_kwargs)
        self.plainTextEdit_mc_kwargs.setObjectName("plainTextEdit_mc_kwargs")
        self.verticalLayout_3.addWidget(self.plainTextEdit_mc_kwargs)
        self.verticalLayout.addWidget(self.groupBox_motion_correction_kwargs)
        self.label_16 = QtWidgets.QLabel(self.dockWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(12)
        font.setBold(True)
        font.setWeight(75)
        self.label_16.setFont(font)
        self.label_16.setObjectName("label_16")
        self.verticalLayout.addWidget(self.label_16)
        self.horizontalLayout_21 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_21.setObjectName("horizontalLayout_21")
        self.label_11 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_11.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.label_11.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_11.setObjectName("label_11")
        self.horizontalLayout_21.addWidget(self.label_11)
        self.comboBoxOutputBitDepth = QtWidgets.QComboBox(self.dockWidgetContents)
        self.comboBoxOutputBitDepth.setObjectName("comboBoxOutputBitDepth")
        self.comboBoxOutputBitDepth.addItem("")
        self.comboBoxOutputBitDepth.addItem("")
        self.comboBoxOutputBitDepth.addItem("")
        self.horizontalLayout_21.addWidget(self.comboBoxOutputBitDepth)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_21.addItem(spacerItem)
        self.verticalLayout.addLayout(self.horizontalLayout_21)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.label = QtWidgets.QLabel(self.dockWidgetContents)
        self.label.setObjectName("label")
        self.horizontalLayout.addWidget(self.label)
        self.spinBoxGSig_filt = QtWidgets.QSpinBox(self.dockWidgetContents)
        self.spinBoxGSig_filt.setObjectName("spinBoxGSig_filt")
        self.horizontalLayout.addWidget(self.spinBoxGSig_filt)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem1)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.horizontalLayout_12 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_12.setObjectName("horizontalLayout_12")
        self.label_9 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_9.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.label_9.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_9.setObjectName("label_9")
        self.horizontalLayout_12.addWidget(self.label_9)
        self.spinboxX = QtWidgets.QSpinBox(self.dockWidgetContents)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.spinboxX.sizePolicy().hasHeightForWidth())
        self.spinboxX.setSizePolicy(sizePolicy)
        self.spinboxX.setMaximum(999)
        self.spinboxX.setObjectName("spinboxX")
        self.horizontalLayout_12.addWidget(self.spinboxX)
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_12.addItem(spacerItem2)
        self.verticalLayout.addLayout(self.horizontalLayout_12)
        self.horizontalLayout_13 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_13.setObjectName("horizontalLayout_13")
        self.label_13 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_13.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_13.setObjectName("label_13")
        self.horizontalLayout_13.addWidget(self.label_13)
        self.spinboxY = QtWidgets.QSpinBox(self.dockWidgetContents)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.spinboxY.sizePolicy().hasHeightForWidth())
        self.spinboxY.setSizePolicy(sizePolicy)
        self.spinboxY.setMaximum(999)
        self.spinboxY.setObjectName("spinboxY")
        self.horizontalLayout_13.addWidget(self.spinboxY)
        spacerItem3 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_13.addItem(spacerItem3)
        self.verticalLayout.addLayout(self.horizontalLayout_13)
        self.horizontalLayout_14 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_14.setObjectName("horizontalLayout_14")
        self.label_6 = QtWidgets.QLabel(self.dockWidgetContents)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_6.sizePolicy().hasHeightForWidth())
        self.label_6.setSizePolicy(sizePolicy)
        self.label_6.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_6.setObjectName("label_6")
        self.horizontalLayout_14.addWidget(self.label_6)
        self.spinboxIterRigid = QtWidgets.QSpinBox(self.dockWidgetContents)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.spinboxIterRigid.sizePolicy().hasHeightForWidth())
        self.spinboxIterRigid.setSizePolicy(sizePolicy)
        self.spinboxIterRigid.setMinimum(1)
        self.spinboxIterRigid.setMaximum(5)
        self.spinboxIterRigid.setObjectName("spinboxIterRigid")
        self.horizontalLayout_14.addWidget(self.spinboxIterRigid)
        spacerItem4 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_14.addItem(spacerItem4)
        self.verticalLayout.addLayout(self.horizontalLayout_14)
        self.label_17 = QtWidgets.QLabel(self.dockWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(12)
        font.setBold(True)
        font.setWeight(75)
        self.label_17.setFont(font)
        self.label_17.setObjectName("label_17")
        self.verticalLayout.addWidget(self.label_17)
        self.horizontalLayout_16 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_16.setObjectName("horizontalLayout_16")
        self.label_15 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_15.setAlignment(QtCore.Qt.AlignCenter)
        self.label_15.setObjectName("label_15")
        self.horizontalLayout_16.addWidget(self.label_15)
        self.spinboxMaxDev = QtWidgets.QSpinBox(self.dockWidgetContents)
        self.spinboxMaxDev.setMaximum(999)
        self.spinboxMaxDev.setProperty("value", 3)
        self.spinboxMaxDev.setObjectName("spinboxMaxDev")
        self.horizontalLayout_16.addWidget(self.spinboxMaxDev)
        spacerItem5 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_16.addItem(spacerItem5)
        self.verticalLayout.addLayout(self.horizontalLayout_16)
        self.verticalLayout_5 = QtWidgets.QVBoxLayout()
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.horizontalLayout_17 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_17.setObjectName("horizontalLayout_17")
        self.label_18 = QtWidgets.QLabel(self.dockWidgetContents)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_18.sizePolicy().hasHeightForWidth())
        self.label_18.setSizePolicy(sizePolicy)
        self.label_18.setAlignment(QtCore.Qt.AlignBottom|QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft)
        self.label_18.setObjectName("label_18")
        self.horizontalLayout_17.addWidget(self.label_18)
        self.spinboxStrides = QtWidgets.QSpinBox(self.dockWidgetContents)
        self.spinboxStrides.setMaximum(500)
        self.spinboxStrides.setProperty("value", 0)
        self.spinboxStrides.setObjectName("spinboxStrides")
        self.horizontalLayout_17.addWidget(self.spinboxStrides)
        spacerItem6 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_17.addItem(spacerItem6)
        self.verticalLayout_5.addLayout(self.horizontalLayout_17)
        self.sliderStrides = QtWidgets.QSlider(self.dockWidgetContents)
        self.sliderStrides.setMinimum(1)
        self.sliderStrides.setMaximum(500)
        self.sliderStrides.setOrientation(QtCore.Qt.Horizontal)
        self.sliderStrides.setObjectName("sliderStrides")
        self.verticalLayout_5.addWidget(self.sliderStrides)
        self.verticalLayout.addLayout(self.verticalLayout_5)
        self.verticalLayout_6 = QtWidgets.QVBoxLayout()
        self.verticalLayout_6.setObjectName("verticalLayout_6")
        self.horizontalLayout_18 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_18.setObjectName("horizontalLayout_18")
        self.label_19 = QtWidgets.QLabel(self.dockWidgetContents)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_19.sizePolicy().hasHeightForWidth())
        self.label_19.setSizePolicy(sizePolicy)
        self.label_19.setAlignment(QtCore.Qt.AlignBottom|QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft)
        self.label_19.setObjectName("label_19")
        self.horizontalLayout_18.addWidget(self.label_19)
        self.spinboxOverlaps = QtWidgets.QSpinBox(self.dockWidgetContents)
        self.spinboxOverlaps.setMaximum(500)
        self.spinboxOverlaps.setProperty("value", 0)
        self.spinboxOverlaps.setObjectName("spinboxOverlaps")
        self.horizontalLayout_18.addWidget(self.spinboxOverlaps)
        spacerItem7 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_18.addItem(spacerItem7)
        self.verticalLayout_6.addLayout(self.horizontalLayout_18)
        self.sliderOverlaps = QtWidgets.QSlider(self.dockWidgetContents)
        self.sliderOverlaps.setMinimum(1)
        self.sliderOverlaps.setMaximum(500)
        self.sliderOverlaps.setOrientation(QtCore.Qt.Horizontal)
        self.sliderOverlaps.setObjectName("sliderOverlaps")
        self.verticalLayout_6.addWidget(self.sliderOverlaps)
        self.verticalLayout.addLayout(self.verticalLayout_6)
        self.btnShowQuilt = QtWidgets.QPushButton(self.dockWidgetContents)
        self.btnShowQuilt.setMaximumSize(QtCore.QSize(100, 16777215))
        self.btnShowQuilt.setCheckable(True)
        self.btnShowQuilt.setObjectName("btnShowQuilt")
        self.verticalLayout.addWidget(self.btnShowQuilt)
        self.horizontalLayout_19 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_19.setObjectName("horizontalLayout_19")
        self.label_10 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_10.setAlignment(QtCore.Qt.AlignCenter)
        self.label_10.setObjectName("label_10")
        self.horizontalLayout_19.addWidget(self.label_10)
        self.spinboxUpsample = QtWidgets.QSpinBox(self.dockWidgetContents)
        self.spinboxUpsample.setMinimum(1)
        self.spinboxUpsample.setMaximum(10)
        self.spinboxUpsample.setProperty("value", 4)
        self.spinboxUpsample.setObjectName("spinboxUpsample")
        self.horizontalLayout_19.addWidget(self.spinboxUpsample)
        spacerItem8 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_19.addItem(spacerItem8)
        self.verticalLayout.addLayout(self.horizontalLayout_19)
        self.checkBoxKeepMemmap = QtWidgets.QCheckBox(self.dockWidgetContents)
        self.checkBoxKeepMemmap.setObjectName("checkBoxKeepMemmap")
        self.verticalLayout.addWidget(self.checkBoxKeepMemmap)
        self.horizontalLayout_20 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_20.setObjectName("horizontalLayout_20")
        self.lineEditNameElastic = QtWidgets.QLineEdit(self.dockWidgetContents)
        self.lineEditNameElastic.setObjectName("lineEditNameElastic")
        self.horizontalLayout_20.addWidget(self.lineEditNameElastic)
        self.btnAddToBatchElastic = QtWidgets.QPushButton(self.dockWidgetContents)
        self.btnAddToBatchElastic.setObjectName("btnAddToBatchElastic")
        self.horizontalLayout_20.addWidget(self.btnAddToBatchElastic)
        self.verticalLayout.addLayout(self.horizontalLayout_20)
        self.line = QtWidgets.QFrame(self.dockWidgetContents)
        self.line.setFrameShape(QtWidgets.QFrame.HLine)
        self.line.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line.setObjectName("line")
        self.verticalLayout.addWidget(self.line)
        self.label_2 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_2.setObjectName("label_2")
        self.verticalLayout.addWidget(self.label_2)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.comboBoxProjectionsOption = QtWidgets.QComboBox(self.dockWidgetContents)
        self.comboBoxProjectionsOption.setObjectName("comboBoxProjectionsOption")
        self.comboBoxProjectionsOption.addItem("")
        self.comboBoxProjectionsOption.addItem("")
        self.comboBoxProjectionsOption.addItem("")
        self.horizontalLayout_2.addWidget(self.comboBoxProjectionsOption)
        self.pushButtonViewProjections = QtWidgets.QPushButton(self.dockWidgetContents)
        self.pushButtonViewProjections.setObjectName("pushButtonViewProjections")
        self.horizontalLayout_2.addWidget(self.pushButtonViewProjections)
        self.verticalLayout.addLayout(self.horizontalLayout_2)
        spacerItem9 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem9)
        DockWidget.setWidget(self.dockWidgetContents)

        self.retranslateUi(DockWidget)
        self.sliderOverlaps.valueChanged['int'].connect(self.spinboxOverlaps.setValue)
        self.sliderStrides.valueChanged['int'].connect(self.spinboxStrides.setValue)
        self.spinboxOverlaps.valueChanged['int'].connect(self.sliderOverlaps.setValue)
        self.spinboxStrides.valueChanged['int'].connect(self.sliderStrides.setValue)
        self.lineEditNameElastic.returnPressed.connect(self.btnAddToBatchElastic.click)
        QtCore.QMetaObject.connectSlotsByName(DockWidget)
        DockWidget.setTabOrder(self.groupBox_motion_correction_kwargs, self.plainTextEdit_mc_kwargs)
        DockWidget.setTabOrder(self.plainTextEdit_mc_kwargs, self.comboBoxOutputBitDepth)
        DockWidget.setTabOrder(self.comboBoxOutputBitDepth, self.spinBoxGSig_filt)
        DockWidget.setTabOrder(self.spinBoxGSig_filt, self.spinboxX)
        DockWidget.setTabOrder(self.spinboxX, self.spinboxY)
        DockWidget.setTabOrder(self.spinboxY, self.spinboxIterRigid)
        DockWidget.setTabOrder(self.spinboxIterRigid, self.spinboxMaxDev)
        DockWidget.setTabOrder(self.spinboxMaxDev, self.spinboxStrides)
        DockWidget.setTabOrder(self.spinboxStrides, self.sliderStrides)
        DockWidget.setTabOrder(self.sliderStrides, self.spinboxOverlaps)
        DockWidget.setTabOrder(self.spinboxOverlaps, self.sliderOverlaps)
        DockWidget.setTabOrder(self.sliderOverlaps, self.btnShowQuilt)
        DockWidget.setTabOrder(self.btnShowQuilt, self.spinboxUpsample)
        DockWidget.setTabOrder(self.spinboxUpsample, self.checkBoxKeepMemmap)
        DockWidget.setTabOrder(self.checkBoxKeepMemmap, self.lineEditNameElastic)
        DockWidget.setTabOrder(self.lineEditNameElastic, self.btnAddToBatchElastic)
        DockWidget.setTabOrder(self.btnAddToBatchElastic, self.comboBoxProjectionsOption)
        DockWidget.setTabOrder(self.comboBoxProjectionsOption, self.pushButtonViewProjections)

    def retranslateUi(self, DockWidget):
        _translate = QtCore.QCoreApplication.translate
        DockWidget.setWindowTitle(_translate("DockWidget", "&CaImAn Motion Correction"))
        self.groupBox_motion_correction_kwargs.setToolTip(_translate("DockWidget", "You can enter additional kwargs to pass for MotionCorrect() instantiation.\n"
"Use single quotes for strings, do not use double quotes."))
        self.groupBox_motion_correction_kwargs.setTitle(_translate("DockWidget", "Use &motion correction kwargs"))
        self.plainTextEdit_mc_kwargs.setToolTip(_translate("DockWidget", "You can enter additional kwargs to pass for CNMF instantiation.\n"
"Use single quotes for strings, do not use double quotes."))
        self.label_16.setToolTip(_translate("DockWidget", "Compute some summary images (correlation and peak to noise ratio)"))
        self.label_16.setText(_translate("DockWidget", "Rigid correction"))
        self.label_11.setText(_translate("DockWidget", "Output bit depth: "))
        self.comboBoxOutputBitDepth.setItemText(0, _translate("DockWidget", "Do not convert"))
        self.comboBoxOutputBitDepth.setItemText(1, _translate("DockWidget", "8"))
        self.comboBoxOutputBitDepth.setItemText(2, _translate("DockWidget", "16"))
        self.label.setText(_translate("DockWidget", "gSig_filt:"))
        self.spinBoxGSig_filt.setToolTip(_translate("DockWidget", "disabled if set to zero"))
        self.label_9.setText(_translate("DockWidget", " max shifts X (pixels): "))
        self.label_13.setText(_translate("DockWidget", " max shifts Y (pixels): "))
        self.label_6.setText(_translate("DockWidget", " iterations for rigid: "))
        self.label_17.setToolTip(_translate("DockWidget", "Compute some summary images (correlation and peak to noise ratio)"))
        self.label_17.setText(_translate("DockWidget", "Elastic correction"))
        self.label_15.setText(_translate("DockWidget", " max deviation from rigid:"))
        self.label_18.setText(_translate("DockWidget", "strides (pixels): "))
        self.label_19.setText(_translate("DockWidget", "overlaps (pixels): "))
        self.btnShowQuilt.setText(_translate("DockWidget", "Show Grid"))
        self.label_10.setText(_translate("DockWidget", " upsample grid:"))
        self.checkBoxKeepMemmap.setText(_translate("DockWidget", "Keep memmap"))
        self.lineEditNameElastic.setPlaceholderText(_translate("DockWidget", "Enter name"))
        self.btnAddToBatchElastic.setText(_translate("DockWidget", "Add to batch"))
        self.label_2.setToolTip(_translate("DockWidget", "View Mean, Max or Std projections of the outputs from selected batch items"))
        self.label_2.setText(_translate("DockWidget", "View projections of selected outputs"))
        self.comboBoxProjectionsOption.setToolTip(_translate("DockWidget", "View Mean, Max or Std projections of the outputs from selected batch items"))
        self.comboBoxProjectionsOption.setItemText(0, _translate("DockWidget", "mean"))
        self.comboBoxProjectionsOption.setItemText(1, _translate("DockWidget", "max"))
        self.comboBoxProjectionsOption.setItemText(2, _translate("DockWidget", "std"))
        self.pushButtonViewProjections.setToolTip(_translate("DockWidget", "View Mean, Max or Std projections of the outputs from selected batch items"))
        self.pushButtonViewProjections.setText(_translate("DockWidget", "View"))


