# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file './control_widget.ui'
#
# Created by: PyQt5 UI code generator 5.9.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_ControlWidget(object):
    def setupUi(self, ControlWidget):
        ControlWidget.setObjectName("ControlWidget")
        ControlWidget.resize(320, 828)
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(ControlWidget)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.label_2 = QtWidgets.QLabel(ControlWidget)
        self.label_2.setObjectName("label_2")
        self.verticalLayout_3.addWidget(self.label_2)
        self.comboBoxDataColumn = ComboBox(ControlWidget)
        self.comboBoxDataColumn.setObjectName("comboBoxDataColumn")
        self.verticalLayout_3.addWidget(self.comboBoxDataColumn)
        self.label_3 = QtWidgets.QLabel(ControlWidget)
        self.label_3.setObjectName("label_3")
        self.verticalLayout_3.addWidget(self.label_3)
        self.comboBoxLabelsColumn = ComboBox(ControlWidget)
        self.comboBoxLabelsColumn.setObjectName("comboBoxLabelsColumn")
        self.verticalLayout_3.addWidget(self.comboBoxLabelsColumn)
        self.label_5 = QtWidgets.QLabel(ControlWidget)
        self.label_5.setObjectName("label_5")
        self.verticalLayout_3.addWidget(self.label_5)
        self.comboBoxDPTCurveColumn = ComboBox(ControlWidget)
        self.comboBoxDPTCurveColumn.setObjectName("comboBoxDPTCurveColumn")
        self.verticalLayout_3.addWidget(self.comboBoxDPTCurveColumn)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.label_6 = QtWidgets.QLabel(ControlWidget)
        self.label_6.setAlignment(QtCore.Qt.AlignCenter)
        self.label_6.setObjectName("label_6")
        self.verticalLayout.addWidget(self.label_6)
        self.doubleSpinBox_vmin = QtWidgets.QDoubleSpinBox(ControlWidget)
        self.doubleSpinBox_vmin.setDecimals(6)
        self.doubleSpinBox_vmin.setObjectName("doubleSpinBox_vmin")
        self.verticalLayout.addWidget(self.doubleSpinBox_vmin)
        self.horizontalLayout_2.addLayout(self.verticalLayout)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout()
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.label_7 = QtWidgets.QLabel(ControlWidget)
        self.label_7.setAlignment(QtCore.Qt.AlignCenter)
        self.label_7.setObjectName("label_7")
        self.verticalLayout_2.addWidget(self.label_7)
        self.doubleSpinBox_vmax = QtWidgets.QDoubleSpinBox(ControlWidget)
        self.doubleSpinBox_vmax.setDecimals(6)
        self.doubleSpinBox_vmax.setObjectName("doubleSpinBox_vmax")
        self.verticalLayout_2.addWidget(self.doubleSpinBox_vmax)
        self.horizontalLayout_2.addLayout(self.verticalLayout_2)
        self.pushButton_set_vmin_vmax = QtWidgets.QPushButton(ControlWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.pushButton_set_vmin_vmax.sizePolicy().hasHeightForWidth())
        self.pushButton_set_vmin_vmax.setSizePolicy(sizePolicy)
        self.pushButton_set_vmin_vmax.setObjectName("pushButton_set_vmin_vmax")
        self.horizontalLayout_2.addWidget(self.pushButton_set_vmin_vmax)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem)
        self.verticalLayout_3.addLayout(self.horizontalLayout_2)
        self.label = QtWidgets.QLabel(ControlWidget)
        self.label.setObjectName("label")
        self.verticalLayout_3.addWidget(self.label)
        self.listWidgetColorMapsData = ColormapListWidget(ControlWidget)
        self.listWidgetColorMapsData.setObjectName("listWidgetColorMapsData")
        self.verticalLayout_3.addWidget(self.listWidgetColorMapsData)
        self.label_4 = QtWidgets.QLabel(ControlWidget)
        self.label_4.setObjectName("label_4")
        self.verticalLayout_3.addWidget(self.label_4)
        self.listWidgetColorMapsLabels = ColormapListWidget(ControlWidget)
        self.listWidgetColorMapsLabels.setObjectName("listWidgetColorMapsLabels")
        self.verticalLayout_3.addWidget(self.listWidgetColorMapsLabels)
        self.checkBoxLiveUpdate = QtWidgets.QCheckBox(ControlWidget)
        font = QtGui.QFont()
        font.setBold(False)
        font.setWeight(50)
        self.checkBoxLiveUpdate.setFont(font)
        self.checkBoxLiveUpdate.setObjectName("checkBoxLiveUpdate")
        self.verticalLayout_3.addWidget(self.checkBoxLiveUpdate)
        self.pushButtonPlot = QtWidgets.QPushButton(ControlWidget)
        self.pushButtonPlot.setMinimumSize(QtCore.QSize(0, 46))
        font = QtGui.QFont()
        font.setPointSize(15)
        font.setBold(True)
        font.setWeight(75)
        self.pushButtonPlot.setFont(font)
        self.pushButtonPlot.setCheckable(False)
        self.pushButtonPlot.setObjectName("pushButtonPlot")
        self.verticalLayout_3.addWidget(self.pushButtonPlot)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.pushButtonSave = QtWidgets.QPushButton(ControlWidget)
        self.pushButtonSave.setMinimumSize(QtCore.QSize(0, 46))
        font = QtGui.QFont()
        font.setPointSize(15)
        font.setBold(True)
        font.setWeight(75)
        self.pushButtonSave.setFont(font)
        self.pushButtonSave.setCheckable(False)
        self.pushButtonSave.setObjectName("pushButtonSave")
        self.horizontalLayout.addWidget(self.pushButtonSave)
        self.pushButtonLoad = QtWidgets.QPushButton(ControlWidget)
        self.pushButtonLoad.setMinimumSize(QtCore.QSize(0, 46))
        font = QtGui.QFont()
        font.setPointSize(15)
        font.setBold(True)
        font.setWeight(75)
        self.pushButtonLoad.setFont(font)
        self.pushButtonLoad.setCheckable(False)
        self.pushButtonLoad.setObjectName("pushButtonLoad")
        self.horizontalLayout.addWidget(self.pushButtonLoad)
        self.verticalLayout_3.addLayout(self.horizontalLayout)
        self.checkBoxShowStimuli = QtWidgets.QCheckBox(ControlWidget)
        self.checkBoxShowStimuli.setObjectName("checkBoxShowStimuli")
        self.verticalLayout_3.addWidget(self.checkBoxShowStimuli)
        self.pushButtonShowHideConsole = QtWidgets.QPushButton(ControlWidget)
        self.pushButtonShowHideConsole.setMinimumSize(QtCore.QSize(0, 38))
        font = QtGui.QFont()
        font.setPointSize(12)
        font.setBold(True)
        font.setWeight(75)
        self.pushButtonShowHideConsole.setFont(font)
        self.pushButtonShowHideConsole.setCheckable(True)
        self.pushButtonShowHideConsole.setObjectName("pushButtonShowHideConsole")
        self.verticalLayout_3.addWidget(self.pushButtonShowHideConsole)

        self.retranslateUi(ControlWidget)
        QtCore.QMetaObject.connectSlotsByName(ControlWidget)
        ControlWidget.setTabOrder(self.comboBoxDataColumn, self.comboBoxLabelsColumn)
        ControlWidget.setTabOrder(self.comboBoxLabelsColumn, self.comboBoxDPTCurveColumn)
        ControlWidget.setTabOrder(self.comboBoxDPTCurveColumn, self.doubleSpinBox_vmin)
        ControlWidget.setTabOrder(self.doubleSpinBox_vmin, self.doubleSpinBox_vmax)
        ControlWidget.setTabOrder(self.doubleSpinBox_vmax, self.pushButton_set_vmin_vmax)
        ControlWidget.setTabOrder(self.pushButton_set_vmin_vmax, self.listWidgetColorMapsData)
        ControlWidget.setTabOrder(self.listWidgetColorMapsData, self.listWidgetColorMapsLabels)
        ControlWidget.setTabOrder(self.listWidgetColorMapsLabels, self.checkBoxLiveUpdate)
        ControlWidget.setTabOrder(self.checkBoxLiveUpdate, self.pushButtonPlot)
        ControlWidget.setTabOrder(self.pushButtonPlot, self.pushButtonSave)
        ControlWidget.setTabOrder(self.pushButtonSave, self.pushButtonLoad)
        ControlWidget.setTabOrder(self.pushButtonLoad, self.checkBoxShowStimuli)
        ControlWidget.setTabOrder(self.checkBoxShowStimuli, self.pushButtonShowHideConsole)

    def retranslateUi(self, ControlWidget):
        _translate = QtCore.QCoreApplication.translate
        ControlWidget.setWindowTitle(_translate("ControlWidget", "Form"))
        self.label_2.setText(_translate("ControlWidget", "Data columns"))
        self.label_3.setText(_translate("ControlWidget", "Labels column"))
        self.label_5.setText(_translate("ControlWidget", "DPT curve column"))
        self.label_6.setText(_translate("ControlWidget", "vmin"))
        self.label_7.setText(_translate("ControlWidget", "vmax"))
        self.pushButton_set_vmin_vmax.setText(_translate("ControlWidget", "Set\n"
"vmin vmax"))
        self.label.setText(_translate("ControlWidget", "Data colormap:"))
        self.label_4.setText(_translate("ControlWidget", "Legend colormap:"))
        self.checkBoxLiveUpdate.setText(_translate("ControlWidget", "Live update from input transmission"))
        self.pushButtonPlot.setText(_translate("ControlWidget", "Plot"))
        self.pushButtonSave.setText(_translate("ControlWidget", "Save"))
        self.pushButtonLoad.setText(_translate("ControlWidget", "Load"))
        self.checkBoxShowStimuli.setText(_translate("ControlWidget", "Show Stimuli"))
        self.pushButtonShowHideConsole.setText(_translate("ControlWidget", "Show/Hide Console"))

from ....pyqtgraphCore.widgets.ComboBox import ComboBox
from ...utils import ColormapListWidget
