# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file './roi_manager_pytemplate.ui'
#
# Created by: PyQt5 UI code generator 5.9.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_DockWidget(object):
    def setupUi(self, DockWidget):
        DockWidget.setObjectName("DockWidget")
        DockWidget.resize(384, 566)
        DockWidget.setMinimumSize(QtCore.QSize(384, 566))
        DockWidget.setFloating(True)
        self.dockWidgetContents = QtWidgets.QWidget()
        self.dockWidgetContents.setObjectName("dockWidgetContents")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.dockWidgetContents)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.tabWidget = QtWidgets.QTabWidget(self.dockWidgetContents)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.tabWidget.sizePolicy().hasHeightForWidth())
        self.tabWidget.setSizePolicy(sizePolicy)
        self.tabWidget.setObjectName("tabWidget")
        self.tabStandard = QtWidgets.QWidget()
        self.tabStandard.setObjectName("tabStandard")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.tabStandard)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.label_3 = QtWidgets.QLabel(self.tabStandard)
        self.label_3.setObjectName("label_3")
        self.horizontalLayout.addWidget(self.label_3)
        self.label_4 = QtWidgets.QLabel(self.tabStandard)
        self.label_4.setObjectName("label_4")
        self.horizontalLayout.addWidget(self.label_4)
        self.verticalLayout_3.addLayout(self.horizontalLayout)
        self.horizontalSliderSpotSize = QtWidgets.QSlider(self.tabStandard)
        self.horizontalSliderSpotSize.setMinimum(1)
        self.horizontalSliderSpotSize.setMaximum(20)
        self.horizontalSliderSpotSize.setPageStep(1)
        self.horizontalSliderSpotSize.setProperty("value", 1)
        self.horizontalSliderSpotSize.setOrientation(QtCore.Qt.Horizontal)
        self.horizontalSliderSpotSize.setObjectName("horizontalSliderSpotSize")
        self.verticalLayout_3.addWidget(self.horizontalSliderSpotSize)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.btnAddROI = QtWidgets.QPushButton(self.tabStandard)
        self.btnAddROI.setObjectName("btnAddROI")
        self.horizontalLayout_3.addWidget(self.btnAddROI)
        self.checkBoxShowAll = QtWidgets.QCheckBox(self.tabStandard)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.checkBoxShowAll.sizePolicy().hasHeightForWidth())
        self.checkBoxShowAll.setSizePolicy(sizePolicy)
        self.checkBoxShowAll.setObjectName("checkBoxShowAll")
        self.horizontalLayout_3.addWidget(self.checkBoxShowAll)
        self.checkBoxLivePlot = QtWidgets.QCheckBox(self.tabStandard)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.checkBoxLivePlot.sizePolicy().hasHeightForWidth())
        self.checkBoxLivePlot.setSizePolicy(sizePolicy)
        self.checkBoxLivePlot.setObjectName("checkBoxLivePlot")
        self.horizontalLayout_3.addWidget(self.checkBoxLivePlot)
        self.btnPlot = QtWidgets.QPushButton(self.tabStandard)
        self.btnPlot.setObjectName("btnPlot")
        self.horizontalLayout_3.addWidget(self.btnPlot)
        self.verticalLayout_3.addLayout(self.horizontalLayout_3)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.label_5 = QtWidgets.QLabel(self.tabStandard)
        self.label_5.setObjectName("label_5")
        self.horizontalLayout_2.addWidget(self.label_5)
        self.radioButton_curve_data = QtWidgets.QRadioButton(self.tabStandard)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.radioButton_curve_data.sizePolicy().hasHeightForWidth())
        self.radioButton_curve_data.setSizePolicy(sizePolicy)
        self.radioButton_curve_data.setObjectName("radioButton_curve_data")
        self.horizontalLayout_2.addWidget(self.radioButton_curve_data)
        self.radioButton_dfof = QtWidgets.QRadioButton(self.tabStandard)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.radioButton_dfof.sizePolicy().hasHeightForWidth())
        self.radioButton_dfof.setSizePolicy(sizePolicy)
        self.radioButton_dfof.setObjectName("radioButton_dfof")
        self.horizontalLayout_2.addWidget(self.radioButton_dfof)
        self.radioButton_spikes = QtWidgets.QRadioButton(self.tabStandard)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.radioButton_spikes.sizePolicy().hasHeightForWidth())
        self.radioButton_spikes.setSizePolicy(sizePolicy)
        self.radioButton_spikes.setObjectName("radioButton_spikes")
        self.horizontalLayout_2.addWidget(self.radioButton_spikes)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem)
        self.verticalLayout_3.addLayout(self.horizontalLayout_2)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.pushButtonImportFromImageJ = QtWidgets.QPushButton(self.tabStandard)
        self.pushButtonImportFromImageJ.setObjectName("pushButtonImportFromImageJ")
        self.horizontalLayout_4.addWidget(self.pushButtonImportFromImageJ)
        self.btnSwitchToManualMode = QtWidgets.QPushButton(self.tabStandard)
        self.btnSwitchToManualMode.setEnabled(False)
        self.btnSwitchToManualMode.setObjectName("btnSwitchToManualMode")
        self.horizontalLayout_4.addWidget(self.btnSwitchToManualMode)
        self.verticalLayout_3.addLayout(self.horizontalLayout_4)
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.label = QtWidgets.QLabel(self.tabStandard)
        self.label.setMaximumSize(QtCore.QSize(16777215, 75))
        self.label.setAlignment(QtCore.Qt.AlignCenter)
        self.label.setObjectName("label")
        self.verticalLayout.addWidget(self.label)
        self.listWidgetROIs = QtWidgets.QListWidget(self.tabStandard)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.listWidgetROIs.sizePolicy().hasHeightForWidth())
        self.listWidgetROIs.setSizePolicy(sizePolicy)
        self.listWidgetROIs.setMinimumSize(QtCore.QSize(50, 0))
        self.listWidgetROIs.setMaximumSize(QtCore.QSize(80, 16777215))
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.listWidgetROIs.setFont(font)
        self.listWidgetROIs.setObjectName("listWidgetROIs")
        self.verticalLayout.addWidget(self.listWidgetROIs)
        self.horizontalLayout_5.addLayout(self.verticalLayout)
        self.verticalLayoutTags = QtWidgets.QVBoxLayout()
        self.verticalLayoutTags.setObjectName("verticalLayoutTags")
        self.label_2 = QtWidgets.QLabel(self.tabStandard)
        self.label_2.setAlignment(QtCore.Qt.AlignCenter)
        self.label_2.setObjectName("label_2")
        self.verticalLayoutTags.addWidget(self.label_2)
        self.listWidgetROITags = QtWidgets.QListWidget(self.tabStandard)
        self.listWidgetROITags.setObjectName("listWidgetROITags")
        self.verticalLayoutTags.addWidget(self.listWidgetROITags)
        self.horizontalLayout_5.addLayout(self.verticalLayoutTags)
        self.verticalLayout_3.addLayout(self.horizontalLayout_5)
        self.lineEditROITag = ROITagLineEdit(self.tabStandard)
        self.lineEditROITag.setObjectName("lineEditROITag")
        self.verticalLayout_3.addWidget(self.lineEditROITag)
        self.btnSetROITag = QtWidgets.QPushButton(self.tabStandard)
        self.btnSetROITag.setObjectName("btnSetROITag")
        self.verticalLayout_3.addWidget(self.btnSetROITag)
        self.tabWidget.addTab(self.tabStandard, "")
        self.tabMetaROI = QtWidgets.QWidget()
        self.tabMetaROI.setObjectName("tabMetaROI")
        self.tabWidget.addTab(self.tabMetaROI, "")
        self.verticalLayout_2.addWidget(self.tabWidget)
        DockWidget.setWidget(self.dockWidgetContents)

        self.retranslateUi(DockWidget)
        self.tabWidget.setCurrentIndex(0)
        self.lineEditROITag.returnPressed.connect(self.btnSetROITag.click)
        self.horizontalSliderSpotSize.valueChanged['int'].connect(self.label_4.setNum)
        QtCore.QMetaObject.connectSlotsByName(DockWidget)
        DockWidget.setTabOrder(self.tabWidget, self.horizontalSliderSpotSize)
        DockWidget.setTabOrder(self.horizontalSliderSpotSize, self.btnAddROI)
        DockWidget.setTabOrder(self.btnAddROI, self.checkBoxShowAll)
        DockWidget.setTabOrder(self.checkBoxShowAll, self.checkBoxLivePlot)
        DockWidget.setTabOrder(self.checkBoxLivePlot, self.btnPlot)
        DockWidget.setTabOrder(self.btnPlot, self.pushButtonImportFromImageJ)
        DockWidget.setTabOrder(self.pushButtonImportFromImageJ, self.btnSwitchToManualMode)
        DockWidget.setTabOrder(self.btnSwitchToManualMode, self.listWidgetROIs)
        DockWidget.setTabOrder(self.listWidgetROIs, self.listWidgetROITags)
        DockWidget.setTabOrder(self.listWidgetROITags, self.lineEditROITag)
        DockWidget.setTabOrder(self.lineEditROITag, self.btnSetROITag)

    def retranslateUi(self, DockWidget):
        _translate = QtCore.QCoreApplication.translate
        DockWidget.setWindowTitle(_translate("DockWidget", "&ROI Manager"))
        self.label_3.setText(_translate("DockWidget", "Spot size:"))
        self.label_4.setText(_translate("DockWidget", "1"))
        self.btnAddROI.setText(_translate("DockWidget", "Add ROI"))
        self.checkBoxShowAll.setText(_translate("DockWidget", "Show all"))
        self.checkBoxLivePlot.setText(_translate("DockWidget", "Live plot"))
        self.btnPlot.setText(_translate("DockWidget", "Plot"))
        self.label_5.setText(_translate("DockWidget", "View:"))
        self.radioButton_curve_data.setToolTip(_translate("DockWidget", "Plot the curve data"))
        self.radioButton_curve_data.setText(_translate("DockWidget", "cur&ve data"))
        self.radioButton_dfof.setToolTip(_translate("DockWidget", "Plot ΔF/F (if available)"))
        self.radioButton_dfof.setText(_translate("DockWidget", "ΔF/F"))
        self.radioButton_spikes.setToolTip(_translate("DockWidget", "Plot spikes (if available)"))
        self.radioButton_spikes.setText(_translate("DockWidget", "spi&kes"))
        self.pushButtonImportFromImageJ.setText(_translate("DockWidget", "Import from ImageJ"))
        self.btnSwitchToManualMode.setText(_translate("DockWidget", "Switch to manual mode"))
        self.label.setText(_translate("DockWidget", "ROIs"))
        self.label_2.setText(_translate("DockWidget", "Tags"))
        self.lineEditROITag.setPlaceholderText(_translate("DockWidget", "Add Tag to ROI Definition"))
        self.btnSetROITag.setText(_translate("DockWidget", "Set ROI tag"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tabStandard), _translate("DockWidget", "Standard ROIs"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tabMetaROI), _translate("DockWidget", "Meta ROIs"))

from ..roi_manager_modules.roi_tag_line_edit import ROITagLineEdit
