# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file './exporter_pytemplate.ui'
#
# Created by: PyQt5 UI code generator 5.9.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_exporter_template(object):
    def setupUi(self, exporter_template):
        exporter_template.setObjectName("exporter_template")
        exporter_template.resize(302, 304)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(exporter_template)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.label = QtWidgets.QLabel(exporter_template)
        self.label.setMaximumSize(QtCore.QSize(75, 16777215))
        self.label.setObjectName("label")
        self.horizontalLayout.addWidget(self.label)
        self.comboBoxFormat = QtWidgets.QComboBox(exporter_template)
        self.comboBoxFormat.setObjectName("comboBoxFormat")
        self.horizontalLayout.addWidget(self.comboBoxFormat)
        self.verticalLayout_2.addLayout(self.horizontalLayout)
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_5.addItem(spacerItem)
        self.labelSlider = QtWidgets.QLabel(exporter_template)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.labelSlider.sizePolicy().hasHeightForWidth())
        self.labelSlider.setSizePolicy(sizePolicy)
        self.labelSlider.setAlignment(QtCore.Qt.AlignCenter)
        self.labelSlider.setObjectName("labelSlider")
        self.horizontalLayout_5.addWidget(self.labelSlider)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_5.addItem(spacerItem1)
        self.verticalLayout_2.addLayout(self.horizontalLayout_5)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.label_4 = QtWidgets.QLabel(exporter_template)
        self.label_4.setObjectName("label_4")
        self.horizontalLayout_2.addWidget(self.label_4)
        self.sliderFPS_Scaling = QtWidgets.QSlider(exporter_template)
        self.sliderFPS_Scaling.setEnabled(False)
        self.sliderFPS_Scaling.setMinimum(1)
        self.sliderFPS_Scaling.setMaximum(100)
        self.sliderFPS_Scaling.setSingleStep(5)
        self.sliderFPS_Scaling.setProperty("value", 10)
        self.sliderFPS_Scaling.setOrientation(QtCore.Qt.Horizontal)
        self.sliderFPS_Scaling.setObjectName("sliderFPS_Scaling")
        self.horizontalLayout_2.addWidget(self.sliderFPS_Scaling)
        self.verticalLayout_2.addLayout(self.horizontalLayout_2)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.label_3 = QtWidgets.QLabel(exporter_template)
        self.label_3.setObjectName("label_3")
        self.horizontalLayout_3.addWidget(self.label_3)
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.radioAuto = QtWidgets.QRadioButton(exporter_template)
        self.radioAuto.setEnabled(False)
        self.radioAuto.setChecked(True)
        self.radioAuto.setObjectName("radioAuto")
        self.verticalLayout.addWidget(self.radioAuto)
        self.radioFromViewer = QtWidgets.QRadioButton(exporter_template)
        self.radioFromViewer.setEnabled(False)
        self.radioFromViewer.setObjectName("radioFromViewer")
        self.verticalLayout.addWidget(self.radioFromViewer)
        self.horizontalLayout_3.addLayout(self.verticalLayout)
        self.verticalLayout_2.addLayout(self.horizontalLayout_3)
        self.checkBoxPseudocolor = QtWidgets.QCheckBox(exporter_template)
        self.checkBoxPseudocolor.setObjectName("checkBoxPseudocolor")
        self.verticalLayout_2.addWidget(self.checkBoxPseudocolor)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.lineEdPath = QtWidgets.QLineEdit(exporter_template)
        self.lineEdPath.setObjectName("lineEdPath")
        self.horizontalLayout_4.addWidget(self.lineEdPath)
        self.btnChoosePath = QtWidgets.QPushButton(exporter_template)
        self.btnChoosePath.setMaximumSize(QtCore.QSize(31, 16777215))
        self.btnChoosePath.setObjectName("btnChoosePath")
        self.horizontalLayout_4.addWidget(self.btnChoosePath)
        self.verticalLayout_2.addLayout(self.horizontalLayout_4)
        self.progressBar = QtWidgets.QProgressBar(exporter_template)
        self.progressBar.setProperty("value", 24)
        self.progressBar.setObjectName("progressBar")
        self.verticalLayout_2.addWidget(self.progressBar)
        self.btnExport = QtWidgets.QPushButton(exporter_template)
        self.btnExport.setObjectName("btnExport")
        self.verticalLayout_2.addWidget(self.btnExport)

        self.retranslateUi(exporter_template)
        QtCore.QMetaObject.connectSlotsByName(exporter_template)
        exporter_template.setTabOrder(self.comboBoxFormat, self.sliderFPS_Scaling)
        exporter_template.setTabOrder(self.sliderFPS_Scaling, self.radioAuto)
        exporter_template.setTabOrder(self.radioAuto, self.radioFromViewer)
        exporter_template.setTabOrder(self.radioFromViewer, self.lineEdPath)
        exporter_template.setTabOrder(self.lineEdPath, self.btnChoosePath)

    def retranslateUi(self, exporter_template):
        _translate = QtCore.QCoreApplication.translate
        exporter_template.setWindowTitle(_translate("exporter_template", "Form"))
        self.label.setText(_translate("exporter_template", "Format"))
        self.labelSlider.setText(_translate("exporter_template", "TextLabel"))
        self.label_4.setText(_translate("exporter_template", "fps scaling"))
        self.sliderFPS_Scaling.setToolTip(_translate("exporter_template", "Speed up or slow down the framerate (only for video and gifs)"))
        self.label_3.setToolTip(_translate("exporter_template", "Lookup table min & max, only for non-tiff files."))
        self.label_3.setText(_translate("exporter_template", "LUT limits:"))
        self.radioAuto.setToolTip(_translate("exporter_template", "From meta-data (if any)"))
        self.radioAuto.setText(_translate("exporter_template", "Auto"))
        self.radioFromViewer.setToolTip(_translate("exporter_template", "From the current min & max as set in the viewer window"))
        self.radioFromViewer.setText(_translate("exporter_template", "From &viewer"))
        self.checkBoxPseudocolor.setText(_translate("exporter_template", "Pseudocolor from Viewer"))
        self.lineEdPath.setPlaceholderText(_translate("exporter_template", "path"))
        self.btnChoosePath.setText(_translate("exporter_template", "..."))
        self.btnExport.setText(_translate("exporter_template", "Export"))

