# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'pca_window.ui'
#
# Created by: PyQt5 UI code generator 5.6
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_PCA_Window(object):
    def setupUi(self, PCA_Window):
        PCA_Window.setObjectName("PCA_Window")
        PCA_Window.setEnabled(True)
        PCA_Window.resize(994, 733)
        self.centralwidget = QtWidgets.QWidget(PCA_Window)
        self.centralwidget.setEnabled(True)
        self.centralwidget.setObjectName("centralwidget")
        self.gridLayout = QtWidgets.QGridLayout(self.centralwidget)
        self.gridLayout.setObjectName("gridLayout")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout()
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.label_4 = QtWidgets.QLabel(self.centralwidget)
        self.label_4.setObjectName("label_4")
        self.horizontalLayout_2.addWidget(self.label_4)
        self.labelFreq = QtWidgets.QLabel(self.centralwidget)
        self.labelFreq.setObjectName("labelFreq")
        self.horizontalLayout_2.addWidget(self.labelFreq)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem)
        self.verticalLayout_4.addLayout(self.horizontalLayout_2)
        self.sliderMaxFreqIndex = QtWidgets.QSlider(self.centralwidget)
        self.sliderMaxFreqIndex.setMinimum(1)
        self.sliderMaxFreqIndex.setMaximum(1000)
        self.sliderMaxFreqIndex.setProperty("value", 400)
        self.sliderMaxFreqIndex.setOrientation(QtCore.Qt.Horizontal)
        self.sliderMaxFreqIndex.setObjectName("sliderMaxFreqIndex")
        self.verticalLayout_4.addWidget(self.sliderMaxFreqIndex)
        self.gridLayout.addLayout(self.verticalLayout_4, 0, 0, 1, 4)
        self.verticalLayout_6 = QtWidgets.QVBoxLayout()
        self.verticalLayout_6.setObjectName("verticalLayout_6")
        self.label_6 = QtWidgets.QLabel(self.centralwidget)
        self.label_6.setMaximumSize(QtCore.QSize(100, 16777215))
        self.label_6.setObjectName("label_6")
        self.verticalLayout_6.addWidget(self.label_6)
        self.comboBoxLog = QtWidgets.QComboBox(self.centralwidget)
        self.comboBoxLog.setMaximumSize(QtCore.QSize(100, 16777215))
        self.comboBoxLog.setObjectName("comboBoxLog")
        self.comboBoxLog.addItem("")
        self.comboBoxLog.addItem("")
        self.comboBoxLog.addItem("")
        self.verticalLayout_6.addWidget(self.comboBoxLog)
        self.gridLayout.addLayout(self.verticalLayout_6, 1, 1, 1, 1)
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.label = QtWidgets.QLabel(self.centralwidget)
        self.label.setObjectName("label")
        self.verticalLayout.addWidget(self.label)
        self.spinBoxTrimStart = QtWidgets.QSpinBox(self.centralwidget)
        self.spinBoxTrimStart.setProperty("value", 1)
        self.spinBoxTrimStart.setObjectName("spinBoxTrimStart")
        self.verticalLayout.addWidget(self.spinBoxTrimStart)
        self.gridLayout.addLayout(self.verticalLayout, 1, 0, 1, 1)
        self.verticalLayout_7 = QtWidgets.QVBoxLayout()
        self.verticalLayout_7.setObjectName("verticalLayout_7")
        self.labelDataColumn = QtWidgets.QLabel(self.centralwidget)
        self.labelDataColumn.setObjectName("labelDataColumn")
        self.verticalLayout_7.addWidget(self.labelDataColumn)
        self.comboBoxDataColumn = QtWidgets.QComboBox(self.centralwidget)
        self.comboBoxDataColumn.setObjectName("comboBoxDataColumn")
        self.verticalLayout_7.addWidget(self.comboBoxDataColumn)
        self.gridLayout.addLayout(self.verticalLayout_7, 1, 2, 1, 1)
        self.verticalLayout_5 = QtWidgets.QVBoxLayout()
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.labelTargetColumn = QtWidgets.QLabel(self.centralwidget)
        self.labelTargetColumn.setObjectName("labelTargetColumn")
        self.verticalLayout_5.addWidget(self.labelTargetColumn)
        self.comboBoxTarget = QtWidgets.QComboBox(self.centralwidget)
        self.comboBoxTarget.setObjectName("comboBoxTarget")
        self.verticalLayout_5.addWidget(self.comboBoxTarget)
        self.gridLayout.addLayout(self.verticalLayout_5, 1, 3, 1, 1)
        spacerItem1 = QtWidgets.QSpacerItem(754, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout.addItem(spacerItem1, 1, 6, 1, 1)
        self.checkBoxLiveUpdate = QtWidgets.QCheckBox(self.centralwidget)
        self.checkBoxLiveUpdate.setObjectName("checkBoxLiveUpdate")
        self.gridLayout.addWidget(self.checkBoxLiveUpdate, 1, 4, 1, 1)
        self.btnPlot = QtWidgets.QPushButton(self.centralwidget)
        self.btnPlot.setObjectName("btnPlot")
        self.gridLayout.addWidget(self.btnPlot, 1, 5, 1, 1)
        PCA_Window.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar(PCA_Window)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 994, 22))
        self.menubar.setObjectName("menubar")
        self.menuView = QtWidgets.QMenu(self.menubar)
        self.menuView.setObjectName("menuView")
        PCA_Window.setMenuBar(self.menubar)
        self.statusbar = QtWidgets.QStatusBar(PCA_Window)
        self.statusbar.setObjectName("statusbar")
        PCA_Window.setStatusBar(self.statusbar)
        self.dockConsole = QtWidgets.QDockWidget(PCA_Window)
        self.dockConsole.setFeatures(QtWidgets.QDockWidget.DockWidgetFloatable|QtWidgets.QDockWidget.DockWidgetMovable)
        self.dockConsole.setObjectName("dockConsole")
        self.dockWidgetContents = QtWidgets.QWidget()
        self.dockWidgetContents.setObjectName("dockWidgetContents")
        self.dockConsole.setWidget(self.dockWidgetContents)
        PCA_Window.addDockWidget(QtCore.Qt.DockWidgetArea(8), self.dockConsole)
        self.actionConsole = QtWidgets.QAction(PCA_Window)
        self.actionConsole.setCheckable(True)
        self.actionConsole.setObjectName("actionConsole")
        self.menuView.addAction(self.actionConsole)
        self.menubar.addAction(self.menuView.menuAction())

        self.retranslateUi(PCA_Window)
        self.sliderMaxFreqIndex.valueChanged['int'].connect(self.labelFreq.setNum)
        self.actionConsole.toggled['bool'].connect(self.dockConsole.setVisible)
        QtCore.QMetaObject.connectSlotsByName(PCA_Window)

    def retranslateUi(self, PCA_Window):
        _translate = QtCore.QCoreApplication.translate
        PCA_Window.setWindowTitle(_translate("PCA_Window", "MainWindow"))
        self.label_4.setText(_translate("PCA_Window", "Max included frequency index:"))
        self.labelFreq.setText(_translate("PCA_Window", "400"))
        self.label_6.setText(_translate("PCA_Window", "Apply log"))
        self.comboBoxLog.setItemText(0, _translate("PCA_Window", "log2"))
        self.comboBoxLog.setItemText(1, _translate("PCA_Window", "log10"))
        self.comboBoxLog.setItemText(2, _translate("PCA_Window", "None"))
        self.label.setText(_translate("PCA_Window", "Trim start index upto"))
        self.labelDataColumn.setText(_translate("PCA_Window", "Data column"))
        self.labelTargetColumn.setText(_translate("PCA_Window", "Target column"))
        self.checkBoxLiveUpdate.setText(_translate("PCA_Window", "Live Update"))
        self.btnPlot.setText(_translate("PCA_Window", "Plot"))
        self.menuView.setTitle(_translate("PCA_Window", "View"))
        self.dockConsole.setWindowTitle(_translate("PCA_Window", "Console"))
        self.actionConsole.setText(_translate("PCA_Window", "Console"))

