# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file './control_widget.ui'
#
# Created by: PyQt5 UI code generator 5.9.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_DockWidgetControls(object):
    def setupUi(self, DockWidgetControls):
        DockWidgetControls.setObjectName("DockWidgetControls")
        DockWidgetControls.resize(307, 801)
        DockWidgetControls.setAllowedAreas(QtCore.Qt.LeftDockWidgetArea)
        self.dockWidgetContents = QtWidgets.QWidget()
        self.dockWidgetContents.setObjectName("dockWidgetContents")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.dockWidgetContents)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.groupBox = QtWidgets.QGroupBox(self.dockWidgetContents)
        self.groupBox.setObjectName("groupBox")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.groupBox)
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.radioButtonDataColumn = QtWidgets.QRadioButton(self.groupBox)
        self.radioButtonDataColumn.setChecked(True)
        self.radioButtonDataColumn.setObjectName("radioButtonDataColumn")
        self.horizontalLayout.addWidget(self.radioButtonDataColumn)
        self.comboBoxDataColumn = QtWidgets.QComboBox(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.comboBoxDataColumn.sizePolicy().hasHeightForWidth())
        self.comboBoxDataColumn.setSizePolicy(sizePolicy)
        self.comboBoxDataColumn.setObjectName("comboBoxDataColumn")
        self.horizontalLayout.addWidget(self.comboBoxDataColumn)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.radioButtonXY = QtWidgets.QRadioButton(self.groupBox)
        self.radioButtonXY.setObjectName("radioButtonXY")
        self.verticalLayout.addWidget(self.radioButtonXY)
        self.horizontalLayout_9 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_9.setObjectName("horizontalLayout_9")
        self.label = QtWidgets.QLabel(self.groupBox)
        self.label.setObjectName("label")
        self.horizontalLayout_9.addWidget(self.label)
        self.comboBoxXColumn = QtWidgets.QComboBox(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.comboBoxXColumn.sizePolicy().hasHeightForWidth())
        self.comboBoxXColumn.setSizePolicy(sizePolicy)
        self.comboBoxXColumn.setObjectName("comboBoxXColumn")
        self.horizontalLayout_9.addWidget(self.comboBoxXColumn)
        self.verticalLayout.addLayout(self.horizontalLayout_9)
        self.horizontalLayout_10 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_10.setObjectName("horizontalLayout_10")
        self.label_8 = QtWidgets.QLabel(self.groupBox)
        self.label_8.setObjectName("label_8")
        self.horizontalLayout_10.addWidget(self.label_8)
        self.comboBoxYColumn = QtWidgets.QComboBox(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.comboBoxYColumn.sizePolicy().hasHeightForWidth())
        self.comboBoxYColumn.setSizePolicy(sizePolicy)
        self.comboBoxYColumn.setObjectName("comboBoxYColumn")
        self.horizontalLayout_10.addWidget(self.comboBoxYColumn)
        self.verticalLayout.addLayout(self.horizontalLayout_10)
        self.horizontalLayout_12 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_12.setObjectName("horizontalLayout_12")
        self.checkBoxLogX = QtWidgets.QCheckBox(self.groupBox)
        self.checkBoxLogX.setObjectName("checkBoxLogX")
        self.horizontalLayout_12.addWidget(self.checkBoxLogX)
        self.checkBoxLogY = QtWidgets.QCheckBox(self.groupBox)
        self.checkBoxLogY.setObjectName("checkBoxLogY")
        self.horizontalLayout_12.addWidget(self.checkBoxLogY)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_12.addItem(spacerItem)
        self.verticalLayout.addLayout(self.horizontalLayout_12)
        self.verticalLayout_2.addWidget(self.groupBox)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.label_2 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_2.setObjectName("label_2")
        self.horizontalLayout_2.addWidget(self.label_2)
        self.comboBoxColorsColumn = QtWidgets.QComboBox(self.dockWidgetContents)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.comboBoxColorsColumn.sizePolicy().hasHeightForWidth())
        self.comboBoxColorsColumn.setSizePolicy(sizePolicy)
        self.comboBoxColorsColumn.setObjectName("comboBoxColorsColumn")
        self.horizontalLayout_2.addWidget(self.comboBoxColorsColumn)
        self.verticalLayout_2.addLayout(self.horizontalLayout_2)
        self.label_7 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_7.setObjectName("label_7")
        self.verticalLayout_2.addWidget(self.label_7)
        self.listWidgetColormap = ColormapListWidget(self.dockWidgetContents)
        self.listWidgetColormap.setObjectName("listWidgetColormap")
        self.verticalLayout_2.addWidget(self.listWidgetColormap)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.label_3 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_3.setObjectName("label_3")
        self.horizontalLayout_3.addWidget(self.label_3)
        self.comboBoxShapesColumn = QtWidgets.QComboBox(self.dockWidgetContents)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.comboBoxShapesColumn.sizePolicy().hasHeightForWidth())
        self.comboBoxShapesColumn.setSizePolicy(sizePolicy)
        self.comboBoxShapesColumn.setObjectName("comboBoxShapesColumn")
        self.horizontalLayout_3.addWidget(self.comboBoxShapesColumn)
        self.verticalLayout_2.addLayout(self.horizontalLayout_3)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.label_4 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_4.setObjectName("label_4")
        self.horizontalLayout_4.addWidget(self.label_4)
        self.comboBoxUUIDColumn = QtWidgets.QComboBox(self.dockWidgetContents)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.comboBoxUUIDColumn.sizePolicy().hasHeightForWidth())
        self.comboBoxUUIDColumn.setSizePolicy(sizePolicy)
        self.comboBoxUUIDColumn.setObjectName("comboBoxUUIDColumn")
        self.horizontalLayout_4.addWidget(self.comboBoxUUIDColumn)
        self.verticalLayout_2.addLayout(self.horizontalLayout_4)
        self.horizontalLayout_11 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_11.setObjectName("horizontalLayout_11")
        self.label_9 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_9.setObjectName("label_9")
        self.horizontalLayout_11.addWidget(self.label_9)
        self.comboBoxDPTCurve = QtWidgets.QComboBox(self.dockWidgetContents)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.comboBoxDPTCurve.sizePolicy().hasHeightForWidth())
        self.comboBoxDPTCurve.setSizePolicy(sizePolicy)
        self.comboBoxDPTCurve.setObjectName("comboBoxDPTCurve")
        self.horizontalLayout_11.addWidget(self.comboBoxDPTCurve)
        self.verticalLayout_2.addLayout(self.horizontalLayout_11)
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.label_5 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_5.setObjectName("label_5")
        self.horizontalLayout_5.addWidget(self.label_5)
        self.spinBoxSpotSize = QtWidgets.QSpinBox(self.dockWidgetContents)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.spinBoxSpotSize.sizePolicy().hasHeightForWidth())
        self.spinBoxSpotSize.setSizePolicy(sizePolicy)
        self.spinBoxSpotSize.setMinimum(1)
        self.spinBoxSpotSize.setMaximum(999)
        self.spinBoxSpotSize.setSingleStep(1)
        self.spinBoxSpotSize.setProperty("value", 10)
        self.spinBoxSpotSize.setObjectName("spinBoxSpotSize")
        self.horizontalLayout_5.addWidget(self.spinBoxSpotSize)
        self.verticalLayout_2.addLayout(self.horizontalLayout_5)
        self.horizontalLayout_6 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_6.setObjectName("horizontalLayout_6")
        self.label_6 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_6.setObjectName("label_6")
        self.horizontalLayout_6.addWidget(self.label_6)
        self.doubleSpinBoxAlpha = QtWidgets.QDoubleSpinBox(self.dockWidgetContents)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.doubleSpinBoxAlpha.sizePolicy().hasHeightForWidth())
        self.doubleSpinBoxAlpha.setSizePolicy(sizePolicy)
        self.doubleSpinBoxAlpha.setMaximum(1.0)
        self.doubleSpinBoxAlpha.setSingleStep(0.05)
        self.doubleSpinBoxAlpha.setProperty("value", 1.0)
        self.doubleSpinBoxAlpha.setObjectName("doubleSpinBoxAlpha")
        self.horizontalLayout_6.addWidget(self.doubleSpinBoxAlpha)
        self.verticalLayout_2.addLayout(self.horizontalLayout_6)
        self.checkBoxLiveUpdateFromFlowchart = QtWidgets.QCheckBox(self.dockWidgetContents)
        self.checkBoxLiveUpdateFromFlowchart.setChecked(True)
        self.checkBoxLiveUpdateFromFlowchart.setObjectName("checkBoxLiveUpdateFromFlowchart")
        self.verticalLayout_2.addWidget(self.checkBoxLiveUpdateFromFlowchart)
        self.pushButtonUpdatePlot = QtWidgets.QPushButton(self.dockWidgetContents)
        self.pushButtonUpdatePlot.setObjectName("pushButtonUpdatePlot")
        self.verticalLayout_2.addWidget(self.pushButtonUpdatePlot)
        self.horizontalLayout_7 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_7.setObjectName("horizontalLayout_7")
        self.pushButtonSave = QtWidgets.QPushButton(self.dockWidgetContents)
        self.pushButtonSave.setObjectName("pushButtonSave")
        self.horizontalLayout_7.addWidget(self.pushButtonSave)
        self.pushButtonLoad = QtWidgets.QPushButton(self.dockWidgetContents)
        self.pushButtonLoad.setObjectName("pushButtonLoad")
        self.horizontalLayout_7.addWidget(self.pushButtonLoad)
        self.verticalLayout_2.addLayout(self.horizontalLayout_7)
        self.horizontalLayout_8 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_8.setObjectName("horizontalLayout_8")
        self.pushButton = QtWidgets.QPushButton(self.dockWidgetContents)
        self.pushButton.setObjectName("pushButton")
        self.horizontalLayout_8.addWidget(self.pushButton)
        self.pushButtonExport = QtWidgets.QPushButton(self.dockWidgetContents)
        self.pushButtonExport.setObjectName("pushButtonExport")
        self.horizontalLayout_8.addWidget(self.pushButtonExport)
        self.verticalLayout_2.addLayout(self.horizontalLayout_8)
        DockWidgetControls.setWidget(self.dockWidgetContents)

        self.retranslateUi(DockWidgetControls)
        QtCore.QMetaObject.connectSlotsByName(DockWidgetControls)
        DockWidgetControls.setTabOrder(self.comboBoxDataColumn, self.comboBoxColorsColumn)
        DockWidgetControls.setTabOrder(self.comboBoxColorsColumn, self.listWidgetColormap)
        DockWidgetControls.setTabOrder(self.listWidgetColormap, self.comboBoxShapesColumn)
        DockWidgetControls.setTabOrder(self.comboBoxShapesColumn, self.comboBoxUUIDColumn)
        DockWidgetControls.setTabOrder(self.comboBoxUUIDColumn, self.spinBoxSpotSize)
        DockWidgetControls.setTabOrder(self.spinBoxSpotSize, self.doubleSpinBoxAlpha)
        DockWidgetControls.setTabOrder(self.doubleSpinBoxAlpha, self.checkBoxLiveUpdateFromFlowchart)
        DockWidgetControls.setTabOrder(self.checkBoxLiveUpdateFromFlowchart, self.pushButtonUpdatePlot)
        DockWidgetControls.setTabOrder(self.pushButtonUpdatePlot, self.pushButtonSave)
        DockWidgetControls.setTabOrder(self.pushButtonSave, self.pushButtonLoad)
        DockWidgetControls.setTabOrder(self.pushButtonLoad, self.pushButton)
        DockWidgetControls.setTabOrder(self.pushButton, self.pushButtonExport)

    def retranslateUi(self, DockWidgetControls):
        _translate = QtCore.QCoreApplication.translate
        DockWidgetControls.setWindowTitle(_translate("DockWidgetControls", "Co&ntrols"))
        self.groupBox.setTitle(_translate("DockWidgetControls", "Data"))
        self.radioButtonDataColumn.setText(_translate("DockWidgetControls", "Data Colu&mn"))
        self.radioButtonXY.setText(_translate("DockWidgetControls", "Separate &X and Y"))
        self.label.setText(_translate("DockWidgetControls", "X:"))
        self.label_8.setText(_translate("DockWidgetControls", "Y:"))
        self.checkBoxLogX.setText(_translate("DockWidgetControls", "log x"))
        self.checkBoxLogY.setText(_translate("DockWidgetControls", "log y"))
        self.label_2.setText(_translate("DockWidgetControls", "Colors based on"))
        self.label_7.setText(_translate("DockWidgetControls", "Choose colormap:"))
        self.label_3.setText(_translate("DockWidgetControls", "Shapes based on"))
        self.label_4.setText(_translate("DockWidgetControls", "UUID column"))
        self.label_9.setText(_translate("DockWidgetControls", "DPT Curve column"))
        self.label_5.setText(_translate("DockWidgetControls", "Spot size"))
        self.label_6.setText(_translate("DockWidgetControls", "Alpha"))
        self.checkBoxLiveUpdateFromFlowchart.setText(_translate("DockWidgetControls", "Live update from flowchart"))
        self.pushButtonUpdatePlot.setText(_translate("DockWidgetControls", "Update Plot"))
        self.pushButtonSave.setText(_translate("DockWidgetControls", "Save"))
        self.pushButtonLoad.setText(_translate("DockWidgetControls", "Load"))
        self.pushButton.setText(_translate("DockWidgetControls", "Export to matplotlib"))
        self.pushButtonExport.setText(_translate("DockWidgetControls", "Export data"))

from ...utils import ColormapListWidget
