# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file './ui_files/cnmf_3d_pytemplate.ui'
#
# Created by: PyQt5 UI code generator 5.12
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_CNMF3D_template(object):
    def setupUi(self, CNMF3D_template):
        CNMF3D_template.setObjectName("CNMF3D_template")
        CNMF3D_template.resize(599, 735)
        CNMF3D_template.setFloating(True)
        self.dockWidgetContents = QtWidgets.QWidget()
        self.dockWidgetContents.setObjectName("dockWidgetContents")
        self.verticalLayout_7 = QtWidgets.QVBoxLayout(self.dockWidgetContents)
        self.verticalLayout_7.setObjectName("verticalLayout_7")
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout()
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.groupBox_seed_components = QtWidgets.QGroupBox(self.dockWidgetContents)
        self.groupBox_seed_components.setCheckable(True)
        self.groupBox_seed_components.setChecked(False)
        self.groupBox_seed_components.setObjectName("groupBox_seed_components")
        self.verticalLayout_9 = QtWidgets.QVBoxLayout(self.groupBox_seed_components)
        self.verticalLayout_9.setObjectName("verticalLayout_9")
        self.lineEdit_seed_components_path = QtWidgets.QLineEdit(self.groupBox_seed_components)
        self.lineEdit_seed_components_path.setObjectName("lineEdit_seed_components_path")
        self.verticalLayout_9.addWidget(self.lineEdit_seed_components_path)
        self.verticalLayout_4.addWidget(self.groupBox_seed_components)
        self.label_6 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_6.setObjectName("label_6")
        self.verticalLayout_4.addWidget(self.label_6)
        self.spinBox_p = QtWidgets.QSpinBox(self.dockWidgetContents)
        self.spinBox_p.setProperty("value", 2)
        self.spinBox_p.setObjectName("spinBox_p")
        self.verticalLayout_4.addWidget(self.spinBox_p)
        self.groupBox_use_patches = QtWidgets.QGroupBox(self.dockWidgetContents)
        self.groupBox_use_patches.setFlat(False)
        self.groupBox_use_patches.setCheckable(True)
        self.groupBox_use_patches.setObjectName("groupBox_use_patches")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.groupBox_use_patches)
        self.verticalLayout.setObjectName("verticalLayout")
        self.label = QtWidgets.QLabel(self.groupBox_use_patches)
        self.label.setObjectName("label")
        self.verticalLayout.addWidget(self.label)
        self.spinBox_rf = QtWidgets.QSpinBox(self.groupBox_use_patches)
        self.spinBox_rf.setMaximum(9999)
        self.spinBox_rf.setSingleStep(5)
        self.spinBox_rf.setProperty("value", 25)
        self.spinBox_rf.setObjectName("spinBox_rf")
        self.verticalLayout.addWidget(self.spinBox_rf)
        self.label_2 = QtWidgets.QLabel(self.groupBox_use_patches)
        self.label_2.setObjectName("label_2")
        self.verticalLayout.addWidget(self.label_2)
        self.spinBox_stride = QtWidgets.QSpinBox(self.groupBox_use_patches)
        self.spinBox_stride.setMaximum(9999)
        self.spinBox_stride.setSingleStep(5)
        self.spinBox_stride.setProperty("value", 15)
        self.spinBox_stride.setObjectName("spinBox_stride")
        self.verticalLayout.addWidget(self.spinBox_stride)
        self.verticalLayout_4.addWidget(self.groupBox_use_patches)
        self.label_3 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_3.setObjectName("label_3")
        self.verticalLayout_4.addWidget(self.label_3)
        self.spinBox_k = QtWidgets.QSpinBox(self.dockWidgetContents)
        self.spinBox_k.setMaximum(9999)
        self.spinBox_k.setProperty("value", 5)
        self.spinBox_k.setObjectName("spinBox_k")
        self.verticalLayout_4.addWidget(self.spinBox_k)
        self.label_4 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_4.setObjectName("label_4")
        self.verticalLayout_4.addWidget(self.label_4)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.spinBox_gSig_x = QtWidgets.QSpinBox(self.dockWidgetContents)
        self.spinBox_gSig_x.setMaximum(999)
        self.spinBox_gSig_x.setProperty("value", 3)
        self.spinBox_gSig_x.setObjectName("spinBox_gSig_x")
        self.horizontalLayout.addWidget(self.spinBox_gSig_x)
        self.spinBox_gSig_y = QtWidgets.QSpinBox(self.dockWidgetContents)
        self.spinBox_gSig_y.setMaximum(999)
        self.spinBox_gSig_y.setProperty("value", 3)
        self.spinBox_gSig_y.setObjectName("spinBox_gSig_y")
        self.horizontalLayout.addWidget(self.spinBox_gSig_y)
        self.spinBox_gSig_z = QtWidgets.QSpinBox(self.dockWidgetContents)
        self.spinBox_gSig_z.setMaximum(999)
        self.spinBox_gSig_z.setProperty("value", 3)
        self.spinBox_gSig_z.setObjectName("spinBox_gSig_z")
        self.horizontalLayout.addWidget(self.spinBox_gSig_z)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.verticalLayout_4.addLayout(self.horizontalLayout)
        self.label_5 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_5.setObjectName("label_5")
        self.verticalLayout_4.addWidget(self.label_5)
        self.doubleSpinBox_merge_threshold = QtWidgets.QDoubleSpinBox(self.dockWidgetContents)
        self.doubleSpinBox_merge_threshold.setMinimum(0.01)
        self.doubleSpinBox_merge_threshold.setMaximum(1.0)
        self.doubleSpinBox_merge_threshold.setSingleStep(0.01)
        self.doubleSpinBox_merge_threshold.setProperty("value", 0.8)
        self.doubleSpinBox_merge_threshold.setObjectName("doubleSpinBox_merge_threshold")
        self.verticalLayout_4.addWidget(self.doubleSpinBox_merge_threshold)
        self.groupBox_cnmf_kwargs = QtWidgets.QGroupBox(self.dockWidgetContents)
        self.groupBox_cnmf_kwargs.setStatusTip("")
        self.groupBox_cnmf_kwargs.setCheckable(True)
        self.groupBox_cnmf_kwargs.setChecked(False)
        self.groupBox_cnmf_kwargs.setObjectName("groupBox_cnmf_kwargs")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.groupBox_cnmf_kwargs)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.plainTextEdit_cnmf_kwargs = QtWidgets.QPlainTextEdit(self.groupBox_cnmf_kwargs)
        self.plainTextEdit_cnmf_kwargs.setObjectName("plainTextEdit_cnmf_kwargs")
        self.verticalLayout_3.addWidget(self.plainTextEdit_cnmf_kwargs)
        self.verticalLayout_4.addWidget(self.groupBox_cnmf_kwargs)
        self.horizontalLayout_3.addLayout(self.verticalLayout_4)
        self.verticalLayout_6 = QtWidgets.QVBoxLayout()
        self.verticalLayout_6.setObjectName("verticalLayout_6")
        self.label_7 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_7.setObjectName("label_7")
        self.verticalLayout_6.addWidget(self.label_7)
        self.doubleSpinBox_rval = QtWidgets.QDoubleSpinBox(self.dockWidgetContents)
        self.doubleSpinBox_rval.setMaximum(1.0)
        self.doubleSpinBox_rval.setSingleStep(0.01)
        self.doubleSpinBox_rval.setProperty("value", 0.7)
        self.doubleSpinBox_rval.setObjectName("doubleSpinBox_rval")
        self.verticalLayout_6.addWidget(self.doubleSpinBox_rval)
        self.label_8 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_8.setObjectName("label_8")
        self.verticalLayout_6.addWidget(self.label_8)
        self.doubleSpinBox_minSNR = QtWidgets.QDoubleSpinBox(self.dockWidgetContents)
        self.doubleSpinBox_minSNR.setSingleStep(0.25)
        self.doubleSpinBox_minSNR.setProperty("value", 2.5)
        self.doubleSpinBox_minSNR.setObjectName("doubleSpinBox_minSNR")
        self.verticalLayout_6.addWidget(self.doubleSpinBox_minSNR)
        self.label_9 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_9.setObjectName("label_9")
        self.verticalLayout_6.addWidget(self.label_9)
        self.doubleSpinBox_decay_time = QtWidgets.QDoubleSpinBox(self.dockWidgetContents)
        self.doubleSpinBox_decay_time.setMinimum(0.25)
        self.doubleSpinBox_decay_time.setProperty("value", 2.0)
        self.doubleSpinBox_decay_time.setObjectName("doubleSpinBox_decay_time")
        self.verticalLayout_6.addWidget(self.doubleSpinBox_decay_time)
        self.groupBox_eval_kwargs = QtWidgets.QGroupBox(self.dockWidgetContents)
        self.groupBox_eval_kwargs.setCheckable(True)
        self.groupBox_eval_kwargs.setChecked(False)
        self.groupBox_eval_kwargs.setObjectName("groupBox_eval_kwargs")
        self.verticalLayout_5 = QtWidgets.QVBoxLayout(self.groupBox_eval_kwargs)
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.plainTextEdit_eval_kwargs = QtWidgets.QPlainTextEdit(self.groupBox_eval_kwargs)
        self.plainTextEdit_eval_kwargs.setObjectName("plainTextEdit_eval_kwargs")
        self.verticalLayout_5.addWidget(self.plainTextEdit_eval_kwargs)
        self.verticalLayout_6.addWidget(self.groupBox_eval_kwargs)
        self.checkBox_refit = QtWidgets.QCheckBox(self.dockWidgetContents)
        self.checkBox_refit.setObjectName("checkBox_refit")
        self.verticalLayout_6.addWidget(self.checkBox_refit)
        self.checkBox_keep_memmap = QtWidgets.QCheckBox(self.dockWidgetContents)
        self.checkBox_keep_memmap.setObjectName("checkBox_keep_memmap")
        self.verticalLayout_6.addWidget(self.checkBox_keep_memmap)
        self.groupBox_use_previous_memmap = QtWidgets.QGroupBox(self.dockWidgetContents)
        self.groupBox_use_previous_memmap.setCheckable(True)
        self.groupBox_use_previous_memmap.setChecked(False)
        self.groupBox_use_previous_memmap.setObjectName("groupBox_use_previous_memmap")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.groupBox_use_previous_memmap)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.lineEdit_memmap_uuid = QtWidgets.QLineEdit(self.groupBox_use_previous_memmap)
        self.lineEdit_memmap_uuid.setObjectName("lineEdit_memmap_uuid")
        self.verticalLayout_2.addWidget(self.lineEdit_memmap_uuid)
        self.verticalLayout_6.addWidget(self.groupBox_use_previous_memmap)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.lineEdit_name = QtWidgets.QLineEdit(self.dockWidgetContents)
        self.lineEdit_name.setObjectName("lineEdit_name")
        self.horizontalLayout_2.addWidget(self.lineEdit_name)
        self.pushButton_add_to_batch = QtWidgets.QPushButton(self.dockWidgetContents)
        self.pushButton_add_to_batch.setObjectName("pushButton_add_to_batch")
        self.horizontalLayout_2.addWidget(self.pushButton_add_to_batch)
        self.verticalLayout_6.addLayout(self.horizontalLayout_2)
        self.horizontalLayout_3.addLayout(self.verticalLayout_6)
        self.verticalLayout_7.addLayout(self.horizontalLayout_3)
        CNMF3D_template.setWidget(self.dockWidgetContents)

        self.retranslateUi(CNMF3D_template)
        self.lineEdit_name.returnPressed.connect(self.pushButton_add_to_batch.click)
        QtCore.QMetaObject.connectSlotsByName(CNMF3D_template)
        CNMF3D_template.setTabOrder(self.groupBox_seed_components, self.lineEdit_seed_components_path)
        CNMF3D_template.setTabOrder(self.lineEdit_seed_components_path, self.spinBox_p)
        CNMF3D_template.setTabOrder(self.spinBox_p, self.groupBox_use_patches)
        CNMF3D_template.setTabOrder(self.groupBox_use_patches, self.spinBox_rf)
        CNMF3D_template.setTabOrder(self.spinBox_rf, self.spinBox_stride)
        CNMF3D_template.setTabOrder(self.spinBox_stride, self.spinBox_k)
        CNMF3D_template.setTabOrder(self.spinBox_k, self.spinBox_gSig_x)
        CNMF3D_template.setTabOrder(self.spinBox_gSig_x, self.spinBox_gSig_y)
        CNMF3D_template.setTabOrder(self.spinBox_gSig_y, self.spinBox_gSig_z)
        CNMF3D_template.setTabOrder(self.spinBox_gSig_z, self.doubleSpinBox_merge_threshold)
        CNMF3D_template.setTabOrder(self.doubleSpinBox_merge_threshold, self.groupBox_cnmf_kwargs)
        CNMF3D_template.setTabOrder(self.groupBox_cnmf_kwargs, self.plainTextEdit_cnmf_kwargs)
        CNMF3D_template.setTabOrder(self.plainTextEdit_cnmf_kwargs, self.doubleSpinBox_rval)
        CNMF3D_template.setTabOrder(self.doubleSpinBox_rval, self.doubleSpinBox_minSNR)
        CNMF3D_template.setTabOrder(self.doubleSpinBox_minSNR, self.doubleSpinBox_decay_time)
        CNMF3D_template.setTabOrder(self.doubleSpinBox_decay_time, self.groupBox_eval_kwargs)
        CNMF3D_template.setTabOrder(self.groupBox_eval_kwargs, self.plainTextEdit_eval_kwargs)
        CNMF3D_template.setTabOrder(self.plainTextEdit_eval_kwargs, self.checkBox_refit)
        CNMF3D_template.setTabOrder(self.checkBox_refit, self.checkBox_keep_memmap)
        CNMF3D_template.setTabOrder(self.checkBox_keep_memmap, self.groupBox_use_previous_memmap)
        CNMF3D_template.setTabOrder(self.groupBox_use_previous_memmap, self.lineEdit_memmap_uuid)
        CNMF3D_template.setTabOrder(self.lineEdit_memmap_uuid, self.lineEdit_name)
        CNMF3D_template.setTabOrder(self.lineEdit_name, self.pushButton_add_to_batch)

    def retranslateUi(self, CNMF3D_template):
        _translate = QtCore.QCoreApplication.translate
        CNMF3D_template.setWindowTitle(_translate("CNMF3D_template", "&CNMF 3D"))
        self.groupBox_seed_components.setToolTip(_translate("CNMF3D_template", "Useful if trying out different paramters for the same input image stack."))
        self.groupBox_seed_components.setTitle(_translate("CNMF3D_template", "Seed components from masks"))
        self.lineEdit_seed_components_path.setPlaceholderText(_translate("CNMF3D_template", "Enter path to .npy file"))
        self.label_6.setText(_translate("CNMF3D_template", "p"))
        self.groupBox_use_patches.setTitle(_translate("CNMF3D_template", "&Use patches"))
        self.label.setText(_translate("CNMF3D_template", "rf"))
        self.label_2.setText(_translate("CNMF3D_template", "stride"))
        self.label_3.setText(_translate("CNMF3D_template", "k"))
        self.label_4.setText(_translate("CNMF3D_template", "gSig, (x, y z)"))
        self.label_5.setText(_translate("CNMF3D_template", "merge threshold"))
        self.groupBox_cnmf_kwargs.setToolTip(_translate("CNMF3D_template", "You can enter additional kwargs to pass for CNMF instantiation.\n"
"Use single quotes for strings, do not use double quotes."))
        self.groupBox_cnmf_kwargs.setTitle(_translate("CNMF3D_template", "Use CNMF &kwargs"))
        self.plainTextEdit_cnmf_kwargs.setToolTip(_translate("CNMF3D_template", "You can enter additional kwargs to pass for CNMF instantiation.\n"
"Use single quotes for strings, do not use double quotes."))
        self.label_7.setText(_translate("CNMF3D_template", "rval threshold"))
        self.label_8.setText(_translate("CNMF3D_template", "min SNR"))
        self.label_9.setText(_translate("CNMF3D_template", "decay time"))
        self.groupBox_eval_kwargs.setToolTip(_translate("CNMF3D_template", "You can enter additional parameters to use for component evaluation.\n"
"Use single quotes for strings, do not use double quotes"))
        self.groupBox_eval_kwargs.setTitle(_translate("CNMF3D_template", "Use e&valuation params"))
        self.plainTextEdit_eval_kwargs.setToolTip(_translate("CNMF3D_template", "You can enter additional parameters to use for component evaluation.\n"
"Use single quotes for strings, do not use double quotes"))
        self.checkBox_refit.setText(_translate("CNMF3D_template", "refit"))
        self.checkBox_keep_memmap.setToolTip(_translate("CNMF3D_template", "Keep the memmap of this batch item.\n"
"If unchecked the memmap will be deleted \n"
"after this batch item is processed unless it \n"
"is required for a later batch item."))
        self.checkBox_keep_memmap.setText(_translate("CNMF3D_template", "Keep memmap of this batch item"))
        self.groupBox_use_previous_memmap.setToolTip(_translate("CNMF3D_template", "Useful if trying out different paramters for the same input image stack."))
        self.groupBox_use_previous_memmap.setTitle(_translate("CNMF3D_template", "Use &memmap from previous batch item"))
        self.lineEdit_memmap_uuid.setPlaceholderText(_translate("CNMF3D_template", "Enter UUID"))
        self.lineEdit_name.setToolTip(_translate("CNMF3D_template", "Enter a name for this batch item"))
        self.lineEdit_name.setPlaceholderText(_translate("CNMF3D_template", "Enter name"))
        self.pushButton_add_to_batch.setText(_translate("CNMF3D_template", "Add to batch"))


